##====================================================================##
##==Purpose: To get ocMOF in the paper.                             ==##
##==         Produce the figures and tables in the supplementary    ==##
##==         appendix.                                              ==##
##==         Generate a matrix of confounder variables:             ==##
##==         sex, batch and ocMOF (or cluster)                      ==##
##==                                                                ==## 
##==Input: *data/clinical_data/TRDB_TIME_VARY_TRAUMA_RPT_20090708   ==##
##==       _154807.xls                                              ==##
##==       Contains MOF (i.e.MARSHALL_NO_GCS) trajectories, which   ==##
##==       means the central nervous nervous system score is        ==##
##==       excluded.                                                ==##  
##==       *data/clinical_data/TRDB_TRAUMA-PATIENT_CLIN_DATA_RPT    ==##
##==       _20090126_195655.xls                                     ==##
##==       Various clinical variables                               ==##
##==       *data/CEL/info.250.csv                                   ==##
##==       Table where the columns are patient ids, microarray ids, ==##
##==       batch ids and hours since injury.                        ==##
##==                                                                ==##
##==Sub-routines:                                                   ==##
##==  *Generate plots for paper                                     ==##
##==  code/1_ocMOF/heatplus.R                                       ==##
##==                                                                ==##
##==Output:                                                         ==##
##==  -xDat: Data frame with rows corresponding to time, columns    ==##
##==         corresponding to patient_id, and the entries the mof   ==##
##==         score.                                                 ==##
##==  data/Rdata/xDat(marshall_nogcs).Rdata                         ==##
##==  -dat.cfr: Data frame with columns corresponding to            ==##
##==            patient_id, cluster (i.e. ocMOF),batch and sex.     ==##
##==  data/Rdata/dat.cfr.Rdata                                      ==##
##==  log/information_paper.log (used the first time)               ==##
##==  -Information provided in the paper                            ==##
##==                                                                ==##
##==Figures for paper:                                              ==##
##==  *Heatmap of MOF                                               ==##
##==  plots/forpaper/Supplementary_Figure3_heatmap.pdf              ==##
##==  *MOF trajectories                                             ==##
##==  plots/forpaper/Supplementary_Figure4_ocMOF.pdf                ==##
##==  *Boxplots to show the ranking of ocMOF                        ==##
##==  plots/forpaper/Main_Figure3_ocMOF_clinicalvar.pdf             ==##
##==                                                                ==##
##==R library: impute, gdata, gplots                                ==##
##==                                                                ==##
##==Updated:                                                        ==##
##==  *Oct 12, 2010:                                                ==##
##==  1. Patient characteristics of the various ocMOF sub-groups    ==##
##==     are expanded to include age and etc. Now this information  ==##
##==     is available at code/6_assessing_data/clinicalinfotable.R  ==##
##==                                                                ==##
##==Prepared by Chuen Seng Tan/John Storey/Keyur Desai              ==##
##==email: chuentan@princeton.edu/jstorey@princeton.edu/            ==##
##==       kdesai@princeton.edu                                     ==##
##====================================================================##


library(impute)
library(gdata)
library(gplots)

##Read in the MOF trajectories
clinDatTV<-read.xls("data/clinical_data/TRDB_TIME_VARY_TRAUMA_RPT_20090708_154807.xls")
valid<-c(1:2,which(colnames(clinDatTV)%in%"MARSHALL_NO_GCS"))
xDat<-clinDatTV[,valid]
##Get the data into wide form
xDat<-reshape(xDat,v.names=colnames(xDat)[-c(1:2)],
              idvar=colnames(xDat)[1],timevar=colnames(xDat)[2],
              direction="wide")
rownames(xDat)<-xDat[,1]
xDat<-xDat[,-1]
xDat<-t(xDat)
rownames(xDat)<-sapply(strsplit(rownames(xDat),split="\\."),function(x) as.numeric(x[2]))
##Restrict the data to 168 patients
p168<-unique(read.table(file="data/CEL/info.250.csv",sep=",")[,"patient.id"])
xDat<-xDat[,colnames(xDat)%in%p168]
save(xDat,file="data/Rdata/xDat(marshall_nogcs).Rdata")


##Drop remove >20 days since >50% patients had missing MOF score
valid<-1:20
perNAVar.valid<-rowSums(is.na(xDat[valid,]))/ncol(xDat[valid,])
print("The percentage of missingness for each day")
summary(perNAVar.valid,digits=3)
xDat20<-xDat[valid,]

##Impute the missing MOF score for MOF score at day 0 to day 20
k<-10
i.xDat20<-impute.knn(xDat20,k=k)


##Dissimilar matrix is the euclidean distance.
if(is.list(i.xDat20)){
  dat<-i.xDat20$data
}else{
  dat<-i.xDat20
}
dist.dat<-dist(t(dat))
##Clustering with agglomeration method set to complete.
hc<-hclust(dist.dat,method="ward")
grp<-cutree(hc,k=5)

##========================================================================##
##==The plots here give us a preliminary ordering of the groups/clusters==##
##==which will require further validation                               ==##
##========================================================================##
pdf("plots/preliminary ordering of clusters of MOF trajectories.pdf")
plot(hc)

##==Read clinical data to validate the groupings
clinDatNV<-read.xls("data/clinical_data/TRDB_TRAUMA-PATIENT_CLIN_DATA_RPT_20090126_195655.xls")
ref<-names(grp)

mat<-clinDatNV[,1]
id<-match(ref,mat)
clinDatNV<-clinDatNV[id,]
if(!all(names(grp)==clinDatNV[,1])) stop("There is something wrong in matching the clinical variables")

##The distribution of the final disposition
##Note that grp==5 has all patients expriencing death
##This is the first time sink is used
sink(file="log/information_paper.log",append=FALSE,split=TRUE)
print("=========================================================")
print("One of the cluster has all patients who experienced death")
print("Suggesting that this cluster has the worst outcome")
print("FINAL_DISP")
print(table(grp,clinDatNV[,"FINAL_DISP"]))
print("=========================================================")
sink(file=NULL)


##To get a rough sense of the ordering of the remaining groups
par(mfrow=c(ceiling(max(grp)/2),2),cex.lab=1.5,cex.axis=1.5,cex.main=2)

    grpCall<-grp
    k0<-max(grpCall)

    par(mfrow=c(2,2))
    ##ICU_DAYS: Days in ICU
    boxplot(icu.days~grp,data.frame(grp=grpCall,icu.days=clinDatNV[,"ICU_DAYS"]),
            main="ward.grp",
            xlab="Grouping label",ylab="Days in ICU",col=c(1:k0)+1)
    tapply(clinDatNV[,"ICU_DAYS"],grpCall,summary)
    meanICU<-tapply(clinDatNV[,"ICU_DAYS"],grpCall,mean)
    medICU<-tapply(clinDatNV[,"ICU_DAYS"],grpCall,quantile,prob=0.5)

    ##DISCHARGE DAY SNC INJ: Days from injury to discahrge
    valid<-which(clinDatNV[,"FINAL_DISP"]!="09: Death")
    boxplot(icu.days~grp,data.frame(grp=grpCall[valid],icu.days=clinDatNV[valid,"DISCHARGE_DAY_SNC_INJ"]),
            main="ward.grp",
            xlab="Grouping label",ylab="Days from injury to discharge ",col=c(1:k0)+1)
    tapply(clinDatNV[valid,"DISCHARGE_DAY_SNC_INJ"],grpCall[valid],summary)
    meanDIS<-tapply(clinDatNV[,"DISCHARGE_DAY_SNC_INJ"],grpCall,mean)
    medDIS<-tapply(clinDatNV[,"DISCHARGE_DAY_SNC_INJ"],grpCall,quantile,prob=0.5)

    plot(meanDIS,meanICU,pch=names(meanICU),col=c(1:k0)+1,
         main="ward.grp",
         sub=paste("Pearson cor:",round(cor(meanDIS,meanICU))))
    plot(medDIS,medICU,pch=names(medICU),col=c(1:k0)+1,
         main="ward.grp",
         sub=paste("Pearson cor:",round(cor(meanDIS,meanICU))))

dev.off()

print(dim(clinDatNV))
print(nrow(clinDatNV)==length(grp))

##The ordering of the ocMOF according to: ocMOF 1 (worst outcome), ..., ocMOF 5 (best outcome)
##This is reverse from the paper but the numbering will change to the paper in the next plot.
grp0<-grp
grp[grp0==2]<-5
grp[grp0==3]<-4
grp[grp0==1]<-3
grp[grp0==4]<-2
grp[grp0==5]<-1
source("code/1_ocMOF/heatplus.R")
pdf("plots/forpaper/Supplementary_Figure3_heatmap.pdf")
nf<-layout(cbind(c(0,2),c(1,3),c(0,4)), height=c(0.5,6),width=c(1,6,1),respect = FALSE)


op<-par(cex=1,cex.axis=1,cex.lab=1,cex.main=1,cex.sub=1)

##Note: Do not reverse order of grp (i.e. 1: bad outcome, 5: good outcome). Somehow the plot looks better.
ocMOF.label<-c("v","iv","iii","ii","i")
ocMOF.col<-c(2,6:3)
ddn<-reorder(as.dendrogram(hc),grp)
colInd <-order.dendrogram(ddn)
all(names(ddn)==names(grp))
colimage<-colorpanel(10,"black","wheat","orange")

	##legend		
    par(mar = c(0, 1, 0, 0))
    plot(0,xlim=c(1,6),ylim=c(0,1),type="n",axes=FALSE,xlab="",ylab="")
    for(i in 1:5){
      rect(i,0.5,i+0.1,0.7,col=ocMOF.col[6-i])
      text(i+0.1,0.55,paste("ocMOF ",ocMOF.label[6-i],sep=""),pos=4,cex=1)
    }

    mm = c(3, 1, 0, 0)
    par(mar = mm)
    cutplot.dendrogram(ddn,  horiz = TRUE, h=70, cluscol = ocMOF.col, xaxt = "n",yaxt="n",
            xaxs = "i", leaflab = "none",ylab="",xlab="",lwd=1,cex.lab=1.6)

    mm = c(4, 1, 1, 0)
	par(mar = mm)
	if(is.list(i.xDat20)){
	  z<-i.xDat20$data[,colInd][1:20,]
	}else{
	  z<-i.xDat20[,colInd][1:20,]
	}
	image(z,col=colimage,axes=FALSE,ylab="")
  	axis(1, at = c(0,0.5,1), labels = c(0,10,20), tick = TRUE,las=1,cex.axis=1);
    mtext("Days since injury",side=1,line=2,cex=1)  	
	box()
	

    mm = c(4, 1, 1, 3)
    par(mar = mm)
	  if(is.list(i.xDat20)){ 
        dummy.x <- seq(min(i.xDat20$data, na.rm = TRUE), max(i.xDat20$data, na.rm = TRUE), 
            length = length(colimage))
	  }else{
        dummy.x <- seq(min(i.xDat20, na.rm = TRUE), max(i.xDat20, na.rm = TRUE), 
            length = length(colimage))	  
	  }	
        dummy.z <- matrix(dummy.x, nrow = 1)
            image(x = 1, y = dummy.x, z = dummy.z, yaxt = "n", xaxt = "n",ylab="",
                col = colimage,xlab="",axes=FALSE,
                zlim=c(dummy.x[1],dummy.x[length(dummy.x)]),
              xlim=c(dummy.x[1],dummy.x[length(dummy.x)]))
  	axis(4, at = c(0,5,10,15), labels = c(0,5,10,15), tick = TRUE,las=1,cex.axis=1);
	box()	  
dev.off()
par(op)
dev.off()


##The ordering of the ocMOF according to paper ocMOF i (best outcome), ..., ocMOF 5 (worst outcome)
grp0<-grp
grp[grp0==1]<-5
grp[grp0==2]<-4
grp[grp0==3]<-3
grp[grp0==4]<-2
grp[grp0==5]<-1
ocMOF.label<-c("i","ii","iii","iv","v")
ocMOF.col<-c(3:6,2)


pdf("plots/forpaper/Supplementary_Figure4_ocMOF.pdf")	  
      varName<-"Marshall Score"
	  time<-c(0,2:28)
	  par(cex=1.4,cex.axis=1.4,cex.lab=1.4,cex.main=1.4,cex.sub=1.4)	  
	  op<-par(mfrow=c(ceiling(max(grp)/2),2))
      for(g in 1:max(grp)){
        plot(0,0,xlim=c(0,20),ylim=range(xDat,na.rm=TRUE),
             main=paste("ocMOF ",ocMOF.label[g],sep=""),
             xlab="Days since injury",ylab=varName,type="n",yaxt="n")
  	    axis(2, at = c(0,5,10,15), labels =c(0,5,10,15), tick = TRUE,las=1,cex=1);
			 
        apply(xDat[,grp==g],2,function(x,y) lines(x,y,col="gray",lty=2,lwd=0.8),x=time)
		     
        if(sum(grp==g)>1){
		  if(g!=5){
            lines(time,rowMeans(xDat[,grp==g],na.rm=TRUE),col=ocMOF.col[g],lwd=1.4)		
		  }else{
		    meanMOF<-rowMeans(xDat[,grp==g],na.rm=TRUE)
			valid<-which(time<=10)
		    lines(time[valid],meanMOF[valid],col=ocMOF.col[g],lwd=1.4)
			valid<-which(time>=10)
		    lines(time[valid],meanMOF[valid],col=ocMOF.col[g],lwd=0.8,lty=2)

		  
		  } 
        }
		  text(18,15,paste("n=",sum(grp==g)))
      }

      plot(0,0,xlim=c(0,20),ylim=range(xDat,na.rm=TRUE),
           xlab="Days since injury",ylab="Mean Marshall",
		   main="Mean trajectory",type="n",yaxt="n")
  	    axis(2, at = c(0,5,10,15), labels =c(0,5,10,15), tick = TRUE,las=1,cex=1);

      for(g in 1:max(grp)){
        if(sum(grp==g)>1){
		  if(g!=5){
            lines(time,rowMeans(xDat[,grp==g],na.rm=TRUE),col=ocMOF.col[g],lwd=1.4)		
		  }else{
		    meanMOF<-rowMeans(xDat[,grp==g],na.rm=TRUE)
			valid<-which(time<=10)
		    lines(time[valid],meanMOF[valid],col=ocMOF.col[g],lwd=1.4)
			valid<-which(time>=10)
		    lines(time[valid],meanMOF[valid],col=ocMOF.col[g],lwd=0.8,lty=2)

		  
		  } 
        }
      }
	  par(op)

dev.off()

##Boxplots to show the ranking of ocMOF
#pdf("plots/forpaper/Supplementary_Figure3_ocMOF_clinicalvar.png",height=480,width=1440)
pdf("plots/forpaper/Figure3_ocMOF_clinicalvar.pdf",height=7,width=21)


      op<-par(mfrow=c(1,3),cex.lab=2.5,cex.axis=2.3,cex.main=2,
	      mar=c(6,8,3,3))
      noGrp<-max(grp)
  	  valid<-which(grp!=0)
      ##DISCHARGE DAY SNC INJ: Days from injury to discahrge
      boxplot(icu.days~grp,data.frame(grp=grp[valid],icu.days=clinDatNV[valid,"DEATH_OR_DISCHARGE_DAY_SNC_INJ"]),
      main="",
      xlab="ocMOF",ylab="",col=ocMOF.col,axes=FALSE)
      mtext("Days from injury to discharge/death",side=2,line=4.5,cex=2.1)            
      axis(2, at = c(0,20,40,60,80,100), labels = c(0,20,40,60,80,100), tick = TRUE,las=1,cex=1.9);
      axis(1, at = c(1:5), labels = ocMOF.label, tick = TRUE,las=1,cex=1.9);
      box()

      tapply(clinDatNV[valid,"DISCHARGE_DAY_SNC_INJ"],grp[valid],summary)
      meanDIS<-c(NA,tapply(clinDatNV[valid,"DISCHARGE_DAY_SNC_INJ"],grp[valid],mean))
      medDIS<-c(NA,tapply(clinDatNV[valid,"DISCHARGE_DAY_SNC_INJ"],grp[valid],quantile,prob=0.5))


      ##ICU_DAYS: Days in ICU
      boxplot(icu.days~grp,data.frame(grp=grp[valid],icu.days=1-pmin(clinDatNV[valid,"ICU_DAYS"]/clinDatNV[valid,"DEATH_OR_DISCHARGE_DAY_SNC_INJ"],1)),
              main="",ylim=c(0,1),
              xlab="ocMOF",ylab="",col=ocMOF.col,axes=FALSE)
      mtext("Proportion of ICU free days",side=2,line=4.5,cex=2.1)            
  	  axis(2, at = c(0,0.2,0.4,0.6,0.8,1), labels = c(0,0.2,0.4,0.6,0.8,1), tick = TRUE,las=1,cex=1.9);
  	  axis(1, at = c(1:5), labels = ocMOF.label, tick = TRUE,las=1,cex=1.9);
      box()
      tapply(clinDatNV[,"ICU_DAYS"],grp,summary)
      meanICU<-tapply(clinDatNV[,"ICU_DAYS"],grp,mean)
      medICU<-tapply(clinDatNV[,"ICU_DAYS"],grp,quantile,prob=0.5)
    

      ##ICU VENTDAYS ICU_TOTAL_VENTILATION_DAYS: ICU total ventilation days
      boxplot(icu.ventdays~grp,data.frame(grp=grp[valid],icu.ventdays=1-pmin(clinDatNV[valid,"ICU_VENTDAYS"]/clinDatNV[valid,"DEATH_OR_DISCHARGE_DAY_SNC_INJ"],1)),
              main="",ylim=c(0,1),
              xlab="ocMOF",ylab="",col=ocMOF.col,axes=FALSE)
      mtext("Proportion of ICU ventilation free days",side=2,line=4.5,cex=2.1)
  	  axis(2, at = c(0,0.2,0.4,0.6,0.8,1), labels = c(0,0.2,0.4,0.6,0.8,1), tick = TRUE,las=1,cex=1.9);
  	  axis(1, at = c(1:5), labels = ocMOF.label, tick = TRUE,las=1,cex=1.9);
      box()
			  
      #tapply(clinDatNV[,"ICU_VENTDAYS"],grp,summary)
      meanVentDay<-tapply(clinDatNV[,"ICU_VENTDAYS"],grp,mean)
      medVentDay<-tapply(clinDatNV[,"ICU_VENTDAYS"],grp,quantile,prob=0.5)
	  par(op)
	  	  
dev.off()

#sink(file="log/information_paper.log",append=TRUE,split=TRUE)
#print("==============================================")
#print("DISCHARGE_DAY_SNC_INJ: ocMOF i to iv (p-value)")
#valid<-which(grp!=5)
#print(cor.test(grp[valid],clinDatNV[valid,"DISCHARGE_DAY_SNC_INJ"],method="spearman")$p.value)
#print("Proportion of ICU free days: ocMOF i to v (p-value)")
#valid<-which(grp!=0)
#print(cor.test(grp[valid],1-pmin(clinDatNV[valid,"ICU_DAYS"]/clinDatNV[valid,"DEATH_OR_DISCHARGE_DAY_SNC_INJ"],1),method="spearman")$p.value)
#print("Proportion of ventilator free days: ocMOF i to v (p-value)")
#valid<-which(grp!=0)
#print(cor.test(grp[valid],1-pmin(clinDatNV[valid,"ICU_VENTDAYS"]/clinDatNV[valid,"DEATH_OR_DISCHARGE_DAY_SNC_INJ"],1),method="spearman")$p.value)
#print("==============================================")
#sink(file=NULL)

##==============================================##
##==Generate a matrix of confounder variables:==##  
##==sex, batch and ocMOF (or cluster)         ==##   
##==============================================##

if(!all(names(grp)==clinDatNV[,1])) stop("There is something wrong in matching the clinical variables")
info.250<-read.table(file="data/CEL/info.250.csv",sep=",")
info.250<-info.250[!duplicated(info.250[,"patient.id"]),]
mat<-info.250[,1]
id<-match(ref,mat)
info.250<-info.250[id,]
if(!all(names(grp)==info.250[,1])) stop("There is something wrong in matching info.250")

dat.cfr<-data.frame(patient_id=clinDatNV[,"PATIENT_ID"],
                    cluster=grp,
                    batch=info.250[,"batch.id"],
                    sex=clinDatNV[,"SEX"])
save(dat.cfr,file="data/Rdata/dat.cfr.Rdata")

##==Updated: Oct 12, 2010
###info for the paper to show the ranking of ocMOF
###This is not the first time sink is used
#sink(file="log/information_paper.log",append=TRUE,split=TRUE)
#print("========================================================")
#print("Information for Supplementary Table 1 (in percentage, %)")
#print("==VENT_ASSOCIATED_PNEUMONIA")
#print("count")
#print(table(grp,clinDatNV[,"VENT_ASSOCIATED_PNEUMONIA"]))
#print("proportion")
#print(round(prop.table(table(grp,clinDatNV[,"VENT_ASSOCIATED_PNEUMONIA"]),margin=1)*100))
#print("==NON_INFECT_COMPLICATION")
#print("count")
#print(table(grp,clinDatNV[,"NON_INFECT_COMPLICATION"]))
#print("proportion")
#print(round(prop.table(table(grp,clinDatNV[,"NON_INFECT_COMPLICATION"]),margin=1)*100))
#print("==SURGICAL_SITE_INFECTION")
#print("count")
#print(table(grp,clinDatNV[,"SURGICAL_SITE_INFECTION"]))
#print("proportion")
#print(round(prop.table(table(grp,clinDatNV[,"SURGICAL_SITE_INFECTION"]),margin=1)*100))
#print("==NOSOCOMIAL_INFECTION")
#print("count")
#print(table(grp,clinDatNV[,"NOSOCOMIAL_INFECTION"]))
#print("proportion")
#print(round(prop.table(table(grp,clinDatNV[,"NOSOCOMIAL_INFECTION"]),margin=1)*100))
#print("==ICU_TRACH")
#print("count")
#print(table(grp,clinDatNV[,"ICU_TRACH"]))
#print("proportion")
#print(round(prop.table(table(grp,clinDatNV[,"ICU_TRACH"]),margin=1)*100))
#print("==Death")
#print("Death day range")
#print(range(clinDatNV[,"DEATH_DAY_SNC_INJ"],na.rm=TRUE))
#print("count")
#print(table(grp,clinDatNV[,"FINAL_DISP"]=="09: Death"))
#print("proportion")
#print(round(prop.table(table(grp,clinDatNV[,"FINAL_DISP"]=="09: Death"),margin=1)*100))
#print("==MOF")
#print("count")
#print(table(grp,no.mof))
#print("proportion")
#print(round(prop.table(table(grp,no.mof),1)*100))
#print("=====================================")
#
#print("=====================================================")
#print("Information about the death days for ocMOF v and iv")
#print("==ocMOF v")
#print(summary(clinDatNV[grp==5,"DEATH_DAY_SNC_INJ"]))
#print("==ocMOF iv")
#tmp<-clinDatNV[grp==4,"DEATH_DAY_SNC_INJ"]
#print(tmp[!is.na(tmp)])
#print("=====================================================")
#
###info for the paper to show the ranking of ocMOF
#print("========================================")
#print("Age and gender information for the paper")
#print("Range of age")
#print(range(clinDatNV[,"AGE"]))
#print("SEX")
#print(table(clinDatNV[,"SEX"]))
#print("========================================")
#sink(file=NULL)

#summary(clinDatNV[,c("VENT_ASSOCIATED_PNEUMONIA","NON_INFECT_COMPLICATION","SURGICAL_SITE_INFECTION","NOSOCOMIAL_INFECTION","ICU_TRACH")])



