##====================================================================##
##==Purpose: Consolidate all the information related to microarrays ==##
##==         across batches.                                        ==##
##==                                                                ==##
##==Input:                                                          ==##
##==  *data/CEL/info.250.csv or info.all.csv                        ==##
##==  Table where the columns are patient ids, microarray ids,      ==##
##==  batch ids and hours since injury.                             ==##
##==  -info.250.csv: Microarrays collected from hour 0 to 250 and   ==##
##==                 with RNA quality >1                            ==##
##==  -info.all.csv: Microarrays collected from hour 0 to 800 and   ==##
##==                 no filtering based on RNA quality              ==##
##==  *txt.info                                                     ==##
##==  Specifying name of the *.txt file which contains the          ==##
##==  microarrays id.                                               ==##
##==  For example, the microarrays for batch 1 is in "data/csv/     ==##
##==  microarrayID250.168.batch1/microarrayID250.168.batch1         ==## 
##==  expression.xls", then txt.info is                             ==##
##==  "microarrayID250.168.batch"                                   ==##
##==                                                                ==##
##==Output:                                                         ==##
##==  "data/Rdata/microarray/",txt.info,".Rdata", for info.250.csv  ==##
##==  -Xraw: A matrix with rows corresponding to probesets, and the ==##
##==         columns the microarrays of patients at different time  ==##
##==         points.                                                ==##
##==  The follow R objects are used to identify the columns of Xraw.==##  
##==  -patient.id: A vector of patients ID                          ==##
##==  -microarray.id: A vector of microarray ID                     ==##
##==  -batch: A vector of batch ID                                  ==##
##==  -hours.si: A vector of the timing which the microarray was    ==##
##==             collected (in hours).                              ==##
##==                                                                ==##
##==Prepared by Chuen Seng Tan/John Storey/Keyur Desai              ==##
##==email: chuentan@princeton.edu/jstorey@princeton.edu/            ==##
##==       kdesai@princeton.edu                                     ==##
##====================================================================##


##======================================================================##
##==Get the raw longititudinal gene expression data from hour 0 to 250==##
##======================================================================##
##Information regarding microarray id, patient id, hours and batch
if(!any(ls()%in%"info.all")){
  ##Microarrays collected from hour 0 to 250 and with RNA quality >1
  info.250<-read.table(file="data/CEL/info.250.csv",sep=",")
}else{
  ##Microarrays collected from hour 0 to 800 and no filtering based on RNA quality
  info.250<-read.table(file="data/CEL/info.all.csv",sep=",")
  days.si<-NULL
}

Xraw<-NULL  ##The microarray matrix
patient.id<-NULL
microarray.id<-NULL
batch<-NULL
hours.si<-NULL
for(i  in 1:4){
	
  print(paste("Batch",i))
	
  ##==Read the microarray data	
  if(!bypassnorm){	
    ##Using microarray that is normalized locally on the machine   
    dat<-read.table(paste("data/csv/",txt.info,i,"/",txt.info,i," expression.xls",sep=""),sep="\t",header=TRUE)
  }else{
	##Using normalized microarray which is used by the paper
    dat<-read.table(paste("data/normarray/",txt.info,i,"/",txt.info,i," expression.xls",sep=""),sep="\t",header=TRUE)
  }
  ##Remove the last column 
  dat<-dat[,-ncol(dat)]
  ##Name the columns of the matrix	
  rownames(dat)<-dat[,1] 
  dat<-dat[,-1]
  if(any(ls()%in%"info.all")){
    ##Microarrays collected from hour 0 to 250 and with RNA quality >1
    ##Consider only the top 500 probesets
	dat<-dat[rownames(dat)%in%top500,]
  }
  	
  ##==Extract out the information related to the microarrays and order it according to the column of dat
  id<-match(colnames(dat),paste("t",info.250[,"microarray.id"],sep=""))
  if(sum(!is.na(id))!=ncol(dat)) stop("something wrong with the data")
  info<-info.250[id,]
  if(!all(paste("t",info[,"microarray.id"],sep="")==colnames(dat))) stop("something wrong with matching")
  if(length(unique(info[,"batch.id"]))!=1 | unique(info[,"batch.id"])!=i) stop("something wrong with matching")	
  
  ##==Consolidate the information across batches
  if(i>1){
    if(!all(rownames(dat)==rownames(Xraw)))  stop("The rows of the expression data are not matching")
  }
  patient.id<-c(patient.id,info[,"patient.id"])	
  microarray.id<-c(microarray.id,paste("t",info[,"microarray.id"],sep=""))
  batch<-c(batch,info[,"batch.id"])
  hours.si<-c(hours.si,info[,"hours.si"])
  Xraw<-cbind(Xraw,as.matrix(dat))
  if(any(ls()%in%"info.all")){
	days.si<-c(days.si,info[,"days.si"])  
  }	

}

tmp<-order(patient.id)
patient.id<-patient.id[tmp]
microarray.id<-microarray.id[tmp]
hours.si<-hours.si[tmp]
batch<-batch[tmp]
Xraw<-Xraw[,tmp]

if(!any(ls()%in%"info.all")){
  save(patient.id,microarray.id,hours.si,batch,Xraw,
  	   file=paste("data/Rdata/microarray/",txt.info,".Rdata",sep=""))
}else{
  days.si<-days.si[tmp]
}


