##====================================================================##
##==Purpose: To generate the adjusted Spearman analysis using 126   ==##
##==         patients                                               ==##
##==                                                                ==##
##==Input:                                                          ==##
##==  *type1                                                        ==##
##==  A string indicating the gene expression of interest           ==## 
##==  e.g.                                                          ==##
##==  txt.info<-"microarrayID250.168.batch"                         ==##
##==  min_hours<-12;                                                ==##
##==  max_hours<-250;                                               ==##
##==  type1<-paste(txt.info," Slope (",min_hours,"-",max_hours,")", ==##
##==  sep="")                                                       ==##
##==  *"data/Rdata/microarray/",type1,".Rdata"                      ==##
##==  gDat, a matrix with columns corresponding to patients and     ==##
##==  rows the probesets. The entries are the related to gene       ==##
##==  expressions.                                                  ==##
##==  *data/Rdata/dat.cfr.Rdata                                     ==##
##==  dat.cfr, a data frame with columns corresponding to           ==## 
##==  patient_id, cluster (i.e. ocMOF),batch and sex.               ==##
##==                                                                ==##
##==Sub-routines:                                                   ==##
##==  *Spearman related functions:                                  ==##
##==  code/3_analysis/analysisfunctions.R                           ==##
##==                                                                ==##
##==Output:                                                         ==## 
##==  *"data/Rdata/res.",type1,".adj.Rdata"                         ==##
##==  Two vectors, resampled p-value (adjPvalsRes),and parametric   ==## 
##==  p-value (adjPvalsPar).                                        ==##
##==  *Ranking of probesets from adjusted Spearman analysis         ==##
##==  "data/csv/res.",type1,"adj.csv"                               ==##
##==  The columns correspond to the probesets ID, resampled p-value,==##
##==  its q-value, and parametric p-value.                          ==##
##==  *log/information_paper.log                                    ==##
##==  Information provided in the paper                             ==##
##==                                                                ==##
##==Figures for paper:                                              ==##
##==  *p-value histogram of adjusted analysis                       ==##
##==  plots/forpaper/Figure_4a_hist_pval.pdf                        ==##
##==                                                                ==## 
##==R library: qvalue                                               ==##
##==                                                                ==##
##==Prepared by Chuen Seng Tan/John Storey/Keyur Desai              ==##
##==email: chuentan@princeton.edu/jstorey@princeton.edu/            ==##
##==       kdesai@princeton.edu                                     ==##
##====================================================================##

library(qvalue)
load(file=paste("data/Rdata/microarray/",type1,".Rdata",sep=""))

##==Filter the microarray data
##Remove the individuals due to quality reasons and death status
gDat<-gDat[,!(as.numeric(colnames(gDat))%in%c(8028773,25993337,33426107))]

##==The load the confounders
load(file=paste("data/Rdata/dat.cfr.Rdata"))

##================================================##
##==Order the patient_id according to microarray==##
##================================================##
ref<-colnames(gDat)
mat<-dat.cfr[,"patient_id"]
id<-match(ref,mat)
cDat<-dat.cfr[id,]
if(!all(cDat[,"patient_id"]==ref)) stop("confounders and gene expression not ordered correctly")

# Create a factor model for the categorical variable, ocMOF (or cluster)
# and a null model with just batch and sex.
mod <- model.matrix( ~ as.factor(cluster)+as.factor(batch)+as.factor(sex),data=cDat)
mod0 <- model.matrix( ~ as.factor(batch)+as.factor(sex),data=cDat)
##The column indices of the primary variables for mod
pvID<-c(2:5)
if(length(unique(cDat[,"cluster"]))<5) stop("ocMOF has less than 5 clusters")

##==================================##
##==The adjusted Spearman analysis==##
##==================================##
print("Performing the adjusted analysis: with batch and gender as confounders")
#Load the required functions for adjusted analysis
source("code/3_analysis/analysisfunctions.R")

##This is the adjusted Spearman analysis with resampled p-values
set.seed(12345)
#Testing workflow
adjPvalsRes<- adjSpearman(gDat,mod,mod0,res.p=200,modID=pvID,mof=cDat[,"cluster"])
#adjPvalsRes<- adjSpearman(gDat,mod,mod0,res.p=2,modID=pvID,mof=cDat[,"cluster"])
adjPvalsPar<- adjSpearman(gDat,mod,mod0,res.p=0,  modID=pvID,mof=cDat[,"cluster"])
qvals<-qvalue(adjPvalsRes)

##Output of the analysis
outputFinal<-data.frame(siggenes=as.character(rownames(gDat)),pvalsRes=adjPvalsRes,qvalsRes=qvals$qvalues,pvalsPar=adjPvalsPar,
                        stringsAsFactors=FALSE)
outputFinal<-outputFinal[order(outputFinal[,2],outputFinal[,4]),]					   
write.table(outputFinal,file=paste("data/csv/res.",type1,"adj.csv",sep=""),sep=",")
#write.table(outputFinal[1:500,],file=paste("data/csv/res.",type1,"adj.top500.csv",sep=""),sep=",")
save(adjPvalsRes,adjPvalsPar,file=paste("data/Rdata/res.",type1,".adj.Rdata",sep=""))

##This not the first time sink is used (i.e. it is at "code/ocMOF/Getocmof.R",)
sink(file="log/information_paper.log",append=TRUE,split=TRUE)

if(type1=="microarrayID250.168.batch (12-250)"){
  print("===================================")
  print("Information for Figure 2b and paper")
}else{
  print("======================================")	
  print("Information for Supplementary Material")
  print(type1)	
}
print("1-pi0 estimate")
print(round((1-qvals$pi0)*100))
print("FDR<1%")
print(sum(qvals$qvalues<0.01))
print("FDR<5%")
print(sum(qvals$qvalues<0.05))
print("FDR<10%")
print(sum(qvals$qvalues<0.1))
print("The FDR for the top 500 probesets (in percentage, %)")
print(round(outputFinal[500,"qvalsRes"]*100,1))
print("===================================")
sink(file=NULL)

##The p-value histogram of the analysis
if(type1=="microarrayID250.168.batch Slope (12-250)"){
  pdf(paste("plots/forpaper/Figure_4a_hist_pval.pdf"))
  op<-par(bg="white",cex=1.5,cex.axis=1,cex.lab=1,cex.main=1,cex.sub=1,mfrow=c(1,1),mar=c(5,5,4,1))
  hist(adjPvalsRes,main="",xlab="P-values",col="gray",yaxt="n",xaxt="n",cex.lab=2)
  axis(1,cex.axis=2)
  axis(2,cex.axis=2)
  dev.off()
  par(op)
  dev.off()
}
