##====================================================================##
##==Purpose: Module approach for the top 3663 probesets from the    ==##
##==         adjusted Spearman analysis using 126 patients.         ==##
##==                                                                ==##
##==Input:                                                          ==##
##==  *data/top3663WPEC/David3663WPEC.txt                           ==##
##==  The gene sets from DAVID: http://david.abcc.ncifcrf.gov/      ==##
##==  *"data/csv/res.",type1,"adj.csv"                              ==##
##==  The columns correspond to the probesets ID, resampled p-value,==##
##==  its q-value, and parametric p-value.                          ==##
##==  *"data/Rdata/microarray/",type1,".Rdata"                      ==##
##==  gDat, a matrix with columns corresponding to patients and     ==##
##==  rows the probests. The entries are the related to gene        ==##
##==  expressions (i.e. slope/WPEC).                                ==##
##==  *data/Rdata/dat.cfr.Rdata                                     ==##
##==  dat.cfr, a data frame with columns corresponding to           ==## 
##==  patient_id, cluster (i.e. ocMOF),batch and sex.               ==##
##==  *"data/Rdata/microarray/",txt.info,".Rdata"                   ==##
##==  -Xraw: A matrix with rows corresponding to probesets, and the ==##
##==         columns the microarrays of patients at different time  ==##
##==         points.                                                ==##
##==  The follow R objects are used to identify the columns of      ==## 
##==  Xraw.                                                         ==##  
##==  -patient.id: A vector of patients ID                          ==##
##==  -microarray.id: A vector of microarray ID                     ==##
##==  -batch: A vector of batch ID                                  ==##
##==  -hours.si: A vector of the timing which the microarray was    ==##
##==             collected (in hours).                              ==##
##==  *data/Rdata/burnmDat.Rdata                                    ==##
##==  burnmDat, a data frame with columns corresponding to          ==##
##==  patient ids, microarray ids, batch/burnt ids and hours since  ==## 
##==  injury                                                        ==##
##==                                                                ==##
##==Sub-routines:                                                   ==##
##==  *Spearman related functions:                                  ==##
##==  code/3_analysis/analysisfunctions.R                           ==##
##==                                                                ==##
##==Figures for paper:                                              ==##
##==  *The dendrogram for clustering the gene sets                  ==##
##==  plots/forpaper/Supplementary_Figure8_dendrogram.pdf           ==##
##==  *The dominant trajectories for each module                    ==##
##==  "plots/forpaper/Supplementary_Figure9to13_Module_",k,".pdf"   ==##
##==                                                                ==##
##==Output:                                                         ==##
##==  *data/Rdata/David3663WPEC.Rdata                               ==##
##==  -info, matrix of the probesets in the genesets. Columns       ==##
##==  correspond to the probeset IDs, the gene names and gene set   ==##
##==  IDs.                                                          ==##
##==  *"data/Rdata/geneGrp_",geneGrp,".Rdata"                       ==##
##==  -dat.pr.all, a matrix where the columns correspond to the     ==##
##==  dominant trajectories of the ocMOF subgroups.                 ==##
##==  -time.all, a vector of the time associated with the rows of   ==##
##==  dat.pt.all.                                                   ==##
##==  *data/Rdata/hc_geneGrp.Rdata                                  ==##
##==  -grp, a vector indicating the cluster membership of gene set. ==##
##==  -no.probeset, a vector indicating the number of probesets in  ==##
##==  the gene set.                                                 ==##
##==  *log/information_paper.log                                    ==##
##==  Information provided in the paper                             ==##
##==                                                                ==##
##==Prepared by Chuen Seng Tan/John Storey/Keyur Desai              ==##
##==email: chuentan@princeton.edu/jstorey@princeton.edu/            ==##
##==       kdesai@princeton.edu                                     ==##
##====================================================================##


##==Read in the gene sets from 3663 probesets
tmp<-read.table("data/top3663WPEC/David3663WPEC.txt",sep="\t",as.is=TRUE)
idG<-grep("Gene Group",tmp[,1])
idA<-grep("AFFYMETRIX_3PRIME_IVT_ID",tmp[,1])

##Row indices to start and end
idStart<-(idA+1)
idEnd<-c((idG[-1]-1),nrow(tmp))

##Extracting the probesets associated with each gene set
info<-NULL
for(i in 1:length(idStart)){
	valid<-c(idStart[i]:idEnd[i])
	pID<-strsplit(tmp[valid,1], split=",")
	each<-sapply(pID,length)
	gene<-rep(tmp[valid,2],times=each)
	pID<-unlist(pID)
	
	if(length(pID)!=length(gene)) stop(i)
	info<-rbind(info,data.frame(probeID=I(pID),gene=I(gene),cluster=i))
}

save(info,file="data/Rdata/David3663WPEC.Rdata")

##==To perform the adjusted Spearman analysis and unadjusted Spearman analysis for selecting consistent probesets
txt.info<-"microarrayID250.168.batch"
min_hours<-12;
max_hours<-250;
constant<-10
type1<-paste(txt.info," Slope (",min_hours,"-",max_hours,")",sep="")
top<-read.table(file=paste("data/csv/res.",type1,"adj.csv",sep=""),sep=",",as.is=TRUE)[1:3663,]

##Need to remove the blank space from the probeset
all(info[,1]%in%top[,1])
id<-which(!info[,1]%in%top[,1])
info[id,1]<-substring(info[id,1],2,nchar(info[id,1]))
if(!all(info[,1]%in%top[,1]))  stop("There is something wrong")

##Information regarding microarray id, patient id, hours and batch, and the probesets for 126 patients
##Getting WPEC and ocMOF
load(file=paste("data/Rdata/microarray/",type1,".Rdata",sep=""))
##Filter the microarray data
##Remove the individuals due to quality reasons and death status
gDat<-gDat[,!(as.numeric(colnames(gDat))%in%c(8028773,25993337,33426107))]
##==Order the patient_id according to microarray
##==Load the MOF score groupings
load(file=paste("data/Rdata/dat.cfr.Rdata"))
ref<-colnames(gDat)

mat<-dat.cfr[,"patient_id"]
id<-match(ref,mat)
cDat<-dat.cfr[id,]
if(!all(cDat[,"patient_id"]==ref)) stop("Something is wrong in matching dat.cfr and gDat")

##Get the expression out
txt.info<-"microarrayID250.168.batch"
load(file=paste("data/Rdata/microarray/",txt.info,".Rdata",sep=""))
id.126<-which(patient.id%in%colnames(gDat))
Xraw<-Xraw[,id.126]
patient.id<-patient.id[id.126]
microarray.id<-microarray.id[id.126]
hours.si<-hours.si[id.126]
ocMOF.id<-cDat[match(patient.id,cDat[,"patient_id"]),"cluster"]
rm(list=c("id.126"))

##Going through all the gene sets and getting the dominant trajectory associated with the gene set
no.probeset<-NULL  ##No of probesets in each gene set
for(geneGrp in 1:max(info[,3])){
  pIds<-info[info[,3]==geneGrp,1]
  Xraw.id<-which(rownames(Xraw)%in%pIds)
  gDatP<-data.frame(id=rep(patient.id,times=length(pIds)),
                    grp=rep(ocMOF.id,times=length(pIds)),
                    time=rep(hours.si,times=length(pIds)),
                    exp=log2(c(t(Xraw[Xraw.id,]))+constant),
                    probeID=rep(rownames(Xraw)[Xraw.id],each=length(patient.id)),
                    stringsAsFactors=FALSE)

  ##=============================================================##
  ##==To scale the expression based on the spearman correlation==##
  ##=============================================================##
  #Functions for Spearman analysis 
  source("code/3_analysis/analysisfunctions.R")
  #pvalues for gene expression and ocMOF association
  tr.cts<-spear.cor.pvalue(gDat[rownames(gDat)%in%pIds,],cDat[,"cluster"],ncol(gDat))	
  ##==Perform the adjusted Spearman analysis
  print(paste("Perform the adjusted Spearman analysis for split"))
  mod <- model.matrix( ~ as.factor(cluster)+as.factor(batch)+as.factor(sex),
                      data=data.frame(cluster=cDat[,"cluster"],batch=cDat[,"batch"],sex=cDat[,"sex"]))
  mod0 <- model.matrix( ~ as.factor(batch)+as.factor(sex),
                      data=data.frame(batch=cDat[,"batch"],sex=cDat[,"sex"]))
  ##The column indices of the primary variables for mod 
  pvID<-c(2:5)
  if(length(unique(cDat[,"cluster"]))<5) stop("ocMOF has less than 5 clusters")
  tr.cts.adj<- adjSpearman(gDat[rownames(gDat)%in%pIds,],mod,mod0,res.p=0,  modID=pvID,mof=cDat[,"cluster"],stat=TRUE)

  ##==Check for consistency of the correlation signs
  signCor<-NULL
  for(i in 1:nrow(tr.cts)){
  	signCor<-rbind(signCor,data.frame(probeset=rownames(tr.cts)[i],
									  sign=sign(tr.cts[i,1]),
									  same=sign(tr.cts[i,1])==sign(tr.cts.adj[i,1])))	
  }
  no.probeset<-c(no.probeset,sum(signCor[,3]))

  ##====================================================================##
  ##==Include only probesets in pIds and patients used in the analysis==##
  ##====================================================================##
  pIds<-pIds[pIds%in%signCor[signCor[,3],1]]
  mDat<-gDatP[gDatP[,"probeID"]%in%pIds,]
  mDat<-mDat[mDat[,1]%in%cDat[,1],]

  ##Remove individuals not in the analysis:
  mDat<-mDat[mDat[,1]%in%ref,]
  ##Restrict to greater than 12 hours and less than 250 hours
  mDat<-mDat[mDat[,3]>=12 & mDat[,3]<=250,]

  ##==Check for consistency of the correlation signs
  for(i in which(signCor[,3])){
	mDat[(mDat[,"probeID"]==signCor[i,1]),"exp"]<-signCor[i,2]*mDat[(mDat[,"probeID"]==signCor[i,1]),"exp"]
  }


  ##=================================================================##
  ##==Details of microarray being burnt on different days: burnmDat==##
  ##==Use to create weights for loess smoothing due to chips being ==##
  ##==burnt on different for the same individual                   ==##
  ##=================================================================##
  load(file="data/Rdata/burnmDat.Rdata")
  ##Replace patientID: 4501723   chip: t20874672   from 16.3 to 14.5 to be consistent with our database
  burnmDat[burnmDat[,"PATIENT_ID"]==4501723 & burnmDat[,"HOURS_SINCE_INJURY"]==16.3,"HOURS_SINCE_INJURY"]<-14.5

  ##Give weights on based on whether the micorarray was burn on same or different batches
  burnmDat<-cbind(burnmDat,wt=rep(NA,nrow(burnmDat)))
  for(i in unique(burnmDat[,"PATIENT_ID"])){
	valid<-which(burnmDat[,"PATIENT_ID"]==i)
	wt<-NULL
	if(length(unique(burnmDat[valid,"BATCH_ID"]))>1){
		for(burnID in as.character(unique(burnmDat[valid,"BATCH_ID"]))){
			
			valid1<-which(as.character(burnmDat[valid,"BATCH_ID"])==burnID) 
			burnmDat[valid[valid1],"wt"]<-length(valid1)^3#/length(valid)
			wt<-c(wt,length(valid1)^3)
			
		}
		burnmDat[valid,"wt"]<-burnmDat[valid,"wt"]/sum(wt)
	}else{
		burnmDat[valid,"wt"]<-1
	}
  }


  ##==========================================##
  ##==Merge the weights with microarray data==##
  ##==========================================##
  tmp<-merge(mDat,burnmDat[,c("PATIENT_ID","HOURS_SINCE_INJURY","wt")],by.x=c(1,3),by.y=c(1,2))
  t1<-tmp[,c("id","time","probeID")];t2<-mDat[,c("id","time","probeID")]  
  if(!all(t1[order(t1[,1],t1[,2],t1[,3]),]==t2[order(t2[,1],t2[,2],t2[,3]),])) stop("Something wrong in merging")

  mDat<-tmp[,c("id","grp","time","exp","probeID","wt")]
  mDat<-mDat[order(mDat[,"id"],mDat[,"probeID"],mDat[,"time"]),]

  if(length(unique(mDat[,1]))!=ncol(gDat)) stop("Duplication is happening")


  ##=====================================================##
  ##==Standardized and center the expression trajectory==##
  ##=====================================================##

  for(i in unique(mDat[,1])){
	valid<-mDat[,1]%in%i
	sd1<-sd(mDat[valid,c(4)])
	
	for(j in pIds){
		valid1<-(mDat[,1]%in%i & mDat[,5]%in%j)
		mDat[valid1,c(4)]<-(mDat[valid1,c(4)]-mean(mDat[valid1,c(4)]))/sd1   	
	}
  }


  ##===================================##
  ##==The labels and colors for ocMOF==##
  ##===================================##
  ##The ordering of the ocMOF according to paper ocMOF i (best outcome), ..., ocMOF 5 (worst outcome)
  ocMOF.label<-c("i","ii","iii","iv","v")
  ocMOF.col<-c(3:6,2)


  ##=================================##
  ##==The loess fit of trajectories==##
  ##=================================## 
  ##Using time-point time-points>=12
  time.all<-sort(unique(mDat[,3]))

  ##==Get the loess fit of each ocMOF subgroup
  dat.pr.all<-NULL
  x.lim<-range(mDat[,3])
  ref.pt<-11
  ##==NOTE: the ocMOF order for mDat[,2] is not reversed (i.e. 1:bad outcome, 5:good outcome)
  for(grp in 1:5){
	
    ##Individuals from group grp
	id<-unique(mDat[mDat[,2]==grp,1])
	
    ##Get the group''s average
	dat<-data.frame(exp=mDat[mDat[,1]%in%id  & mDat[,3]>=12,4],time=mDat[mDat[,1]%in%id  & mDat[,3]>=12,3]) 
	dat.lo <- loess(exp ~ time, dat,weights=mDat[mDat[,1]%in%id  & mDat[,3]>=12,6]^2,span=0.97)
	dat.pr<-predict(dat.lo, data.frame(time = time.all[time.all>=12]))
	dat.pr.all<-cbind(dat.pr.all,dat.pr)
    
  }

  save(dat.pr.all,time.all,file=paste("data/Rdata/geneGrp_",geneGrp,".Rdata",sep=""))

#  ##The range of the y-axis for the first 100 hours
#  zoom.reg.all<-NULL
#  zoom.reg<-NULL
#  for(grp in 1:5){
#	zoom.reg<-range(zoom.reg,dat.pr.all[time.all<=100,grp]-dat.pr.all[ref.pt,grp],na.rm=TRUE)
#	zoom.reg.all<-range(zoom.reg.all,dat.pr.all[,grp]-dat.pr.all[ref.pt,grp],na.rm=TRUE)
#
#  }
#  time.reg<-range(time.all[time.all>=time.all[ref.pt] & time.all<=100])
#  y.lim<-zoom.reg.all#range(mDat[,4])
#
#  pdf(paste("plots/geneGrp_",geneGrp,".pdf",sep=""),width=7*3,height=7*2,pointsize=12)
#  par(mfrow=c(2,3))
#  ##==Plot the loess fit/dominant trajectory of each ocMOF group from 12-250 hours
#  count<-1
#  no.pts<-NULL
#  for(grp in 1:5){  
#    ##Individuals from group grp
#	id<-unique(mDat[mDat[,2]==grp,1])
#    ##Plot each individual''s gene trajectory profile  
#	plot(0,ylim=y.lim,xlim=x.lim,xlab="Hours since injury",ylab="Standardized gene expression",
#		 axe=FALSE,type="n",main=paste("ocMOF ",ocMOF.label[grp],sep=""))
#	
#	for(i in id){
#		valid<-(mDat[,1]%in%i & mDat[,3]>=12)
#		no.pts<-rbind(no.pts,c(i,sum(valid),grp))
#		
#		for(j in pIds){
#			valid1<-(mDat[,5]%in%j & mDat[,3]>=12)
#			lines(mDat[valid & valid1,c(3)],mDat[valid & valid1,c(4)]-dat.pr.all[ref.pt,grp],col=ifelse(grp%in%c(1:3),gray(0.6),gray(0.4)),lty=1,lwd=0.18)#col=gray(0.8),lty=3,lwd=0.8)
#			
#		}  	
#	}
#    ##Plot the group''s average
#	valid<-time.all>=time.all[ref.pt]
#	lines(time.all[valid],dat.pr.all[valid,grp]-dat.pr.all[ref.pt,grp],col=ocMOF.col[grp],lty=1,lwd=5)
#	box()
#	axis(1, at = c(50,100,150,200), labels = c(50,100,150,200), tick = TRUE,las=1,cex.axis=1,lwd=0,lwd.ticks=1);
#
#	rect(time.reg[1], zoom.reg[1],time.reg[2], zoom.reg[2],col=NA,border = 1, lty=2, lwd=2)
#	count<-count+1
#	
#  }
#
#  ##==Plot the loess fit/dominant trajectory of each ocMOF group from 12-100 hours
#  max.time<-100
#  plot(0,ylim=zoom.reg,xlim=c(x.lim[1],max.time),main="All",
#       xlab="Hours since injury",ylab="Standardized gene expression",type="n",axes=FALSE)
#  rect(40, y.lim[1]-10,80, y.lim[2]+10,col=gray(0.9),border = NA)
#  for(grp in 1:5){
#	valid<-which(time.all>=time.all[ref.pt])
#	lines(time.all[valid],dat.pr.all[valid,grp]-dat.pr.all[valid[1],grp],col=ocMOF.col[grp],lwd=5)
#  }
#  axis(1, at = c(25,50,75,100), labels = c(25,50,75,100), tick = TRUE,las=1,cex.axis=1,lwd=0,lwd.ticks=1);
#  box(lty=2,lwd=2)
#  dev.off()

  print(geneGrp)	
	
}

ref.pt<-11
datall<-NULL
dat1<-dat2<-dat3<-dat4<-dat5<-NULL
for(geneGrp in 1:54){
	load(file=paste("data/Rdata/geneGrp_",geneGrp,".Rdata",sep=""))
	for(i in 1:ncol(dat.pr.all)){
		dat.pr.all[,i]<-dat.pr.all[,i]-dat.pr.all[ref.pt,i]
		tmp<-get(paste("dat",i,sep=""))
		assign(paste("dat",i,sep=""),cbind(tmp,dat.pr.all[,i]-dat.pr.all[ref.pt,i]))
	}	
	datall<-cbind(datall,c(dat.pr.all))
}

cor1<-cor(dat1,use="complete.obs")
cor2<-cor(dat2,use="complete.obs")
cor3<-cor(dat3,use="complete.obs")
cor4<-cor(dat4,use="complete.obs")
cor5<-cor(dat5,use="complete.obs")

mcor<-(cor1+cor2+cor3+cor4+cor5)/5
scor<-(sign(cor1)==sign(cor2) & sign(cor1)==sign(cor3) & sign(cor1)==sign(cor4) & sign(cor1)==sign(cor5))
mcor[!scor]<-min(0,mcor[scor])

##==Using correlation
dist.dat<-as.dist(1-mcor)
##Clustering with agglomeration method set to complete.
hc<-hclust(dist.dat,method="ward")
grp<-cutree(hc,k=5)
save(grp,no.probeset,file="data/Rdata/hc_geneGrp.Rdata")

pdf("plots/forpaper/Supplementary_Figure8_dendrogram.pdf")
plot(hc,xlab="Gene sets",sub="",ylab="",main="")
dev.off()

clust.dist<-NULL
for(k in 1:5){
	id<-which(grp==k)
	tmp<-1-mcor[id,id]
	clust.dist<-c(clust.dist,mean(tmp[lower.tri(tmp, diag = FALSE)]))	
}

sink(file="log/information_paper.log",append=TRUE,split=TRUE)
print("============================================================")
print("The total number of probesets from DAVID (without filtering)")
print(nrow(info))
print("Number of gene sets in each module 1 to 5:")
print(table(grp))
print("Number of probesets in each module 1 to 5:")
for(k in 1:5){
  print(k)
  print(sum(no.probeset[grp==k]))
}
print("The total number of probesets from DAVID (with filtering)")
print(sum(no.probeset))
print("Ordering of the modules 1 to 5 into A to E")
print(order(clust.dist))
print("============================================================")
sink(file=NULL)

##Getting the ranking and labeling of the modules in terms of alphabets
rABCDE<-rank(clust.dist)
modABCDE<-c("A","B","C","D","E")

##==========================================##
##==Plot the supplementary Figures 6 to 10==##
##==========================================##
##Get the scaled and centered data
datall<-NULL
for(geneGrp in 1:54){
	load(file=paste("data/Rdata/geneGrp_",geneGrp,".Rdata",sep=""))
	dat.pr.all<-scale(dat.pr.all)	
	for(i in 1:ncol(dat.pr.all)){
		dat.pr.all[,i]<-dat.pr.all[,i]-dat.pr.all[ref.pt,i]
	}	
	datall<-cbind(datall,c(dat.pr.all))
}

for(k in 1:5){
	if(!all(cor(datall[!apply(is.na(datall),1,all),grp==k])>0)) stop("some sign differences")
	ref.pt<-11
        ##The group of interest
	dat<-NULL
	for(geneGrp in which(grp==k)){
		load(file=paste("data/Rdata/geneGrp_",geneGrp,".Rdata",sep=""))
		dat.pr.all<-scale(dat.pr.all)
		for(i in 1:ncol(dat.pr.all)){
			dat.pr.all[,i]<-dat.pr.all[,i]-dat.pr.all[ref.pt,i]
		}	
		dat<-rbind(dat,cbind(exp=c(dat.pr.all),time=rep(time.all,time=5),ocMOF=rep(1:5,each=nrow(dat.pr.all)),geneGrp=geneGrp))
	}
	
        ##Get the mean expression
	mean.exp<-matrix(apply(datall[,which(grp==k)],1,mean),ncol=5)
	
        ##The ordering of the ocMOF according to paper ocMOF i (best outcome), ..., ocMOF 5 (worst outcome)
	ocMOF.label<-c("i","ii","iii","iv","v")
	ocMOF.col<-c(3:6,2)
	
	pdf(paste("plots/forpaper/Supplementary_Figure9to13_Module_",modABCDE[rABCDE[k]],"_Fig",8+rABCDE[k],".pdf",sep=""),
            width=3*3,height=3*2,pointsize=12)
	par(mfrow=c(2,3))
	label.info<-c("a","b","c","d","e")
	for(i in 1:5){
		plot(0,xlim=range(dat[,"time"]),ylim=range(dat[,"exp"],na.rm=TRUE),
			 xlab="Hours since injury",ylab="Average std. expression",
			 axe=FALSE,type="n")#,main=paste("ocMOF ",ocMOF.label[i],sep=""))
		for(j in unique(dat[,"geneGrp"])){
			valid<-which(dat[,"ocMOF"]==i & dat[,"geneGrp"]==j)
			lines(dat[valid,"time"],dat[valid,"exp"],col="gray",lwd=1,lty=1)
		}
		lines(dat[valid,"time"],mean.exp[,i],col=ocMOF.col[i],lwd=3)
		box()
		axis(1, at = c(50,100,150,200), labels = c(50,100,150,200), tick = TRUE,las=1,cex.axis=1,lwd=0,lwd.ticks=1);
		mtext(label.info[i],side=3,adj=0,font=2)#,cex=1.5)
	}
	n.valid<-length(valid)
	
        ##==Plot the loess fit/dominant trajectory of each ocMOF group
	y.lim<-range(mean.exp,na.rm=TRUE)
	x.lim<-range(dat[,"time"])
	plot(0,ylim=y.lim,xlim=x.lim,
		 xlab="Hours since injury",ylab="Dominant expression",type="n",axes=FALSE)
	for(i in 1:5){
		valid<-which(time.all>=time.all[ref.pt])
		lines(time.all[valid],mean.exp[valid,i],col=ocMOF.col[i],lwd=3)
	}
	axis(1, at = c(50,100,150,200), labels = c(50,100,150,200), tick = TRUE,las=1,cex.axis=1,lwd=0,lwd.ticks=1);
        box()
	mtext("f",side=3,adj=0,font=2)#,cex=1.5)
	dev.off()
}
