##====================================================================##
##==Purpose: To get the probesets of the top 6 canonical pathways   ==##
##==         after filtering out biologically inconsistent ones     ==##
##==         To get the probesets from DAVID on MHC-II              ==## 
##==         To get the expression trajectories associated with the ==##
##==         gene sets                                              ==##
##==                                                                ==##
##==Input:                                                          ==##
##==  These are used to identify the probsets in the gene sets      ==##
##==  *type1                                                        ==##
##==  A string indicating the gene expression of interest           ==## 
##==  e.g.                                                          ==##
##==  txt.info<-"microarrayID250.168.batch"                         ==##
##==  min_hours<-12;                                                ==##
##==  max_hours<-250;                                               ==##
##==  type1<-paste(txt.info," Slope (",min_hours,"-",max_hours,")", ==##
##==  sep="")                                                       ==##
##==  *"data/Rdata/microarray/",type1,".Rdata"                      ==##
##==  gDat, a matrix with columns corresponding to patients and     ==##
##==  rows the probests. The entries are the related to gene        ==##
##==  expressions (i.e. slope/WPEC).                                ==##
##==  *data/Rdata/dat.cfr.Rdata                                     ==##
##==  dat.cfr, a data frame with columns corresponding to           ==## 
##==  patient_id, cluster (i.e. ocMOF),batch and sex.               ==##
##==  *"data/csv/res.",type1,"adj.csv"                              ==##
##==  The columns correspond to the probesets ID, resampled p-value,==##
##==  its q-value, and parametric p-value.                          ==##
##==  *Data from Ingenuity                                          ==##
##==  data/Ingenuity/canonical pathway.Top500.slope.txt             ==##
##==  data/Ingenuity/Top500slopeMapped.txt                          ==##
##==  *txt.info                                                     ==##
##==  Specifying name of the *.txt file which contains the          ==##        
##==  microarrays id. For example, the microarrays for batch is in  ==##
##==  "microarrayID250.168.batch1.txt", then txt.info is            ==##
##==  "microarrayID250.168.batch"                                   ==##
##==  These are used to extract the expression trajectories of the  ==## 
##==  gene sets                                                     ==##
##==  *data/Rdata/microarray/txt.info.Rdata                         ==##
##==  -Xraw: A matrix with rows corresponding to probesets, and the ==##
##==         columns the microarrays of patients at different time  ==##
##==         points.                                                ==##
##==  The follow R objects are used to identify the columns of Xraw.==##  
##==  -patient.id: A vector of patients ID                          ==##
##==  -microarray.id: A vector of microarray ID                     ==##
##==  -batch: A vector of batch ID                                  ==##
##==  -hours.si: A vector of the timing which the microarray was    ==##
##==             collected (in hours).                              ==##
##==                                                                ==##
##==Output:                                                         ==##
##==  *Filtered set of probsets associated with each gene sets      ==##
##==  data/Rdata/probesets.CanonicalPathway.Rdata                   ==##
##==  These R objects: pIds.mhc2, pIds.p38, pIds.tlr, pIds.il6,     ==##
##==  and pIds.top5, contain the probesets id associated with the   ==##
##==  each gene set.                                                ==##
##==  *Expression trajectories of all 126 patients for the gene sets==##
##==  data/Rdata/MHC2.Rdata                                         ==##
##==  data/Rdata/P38.Rdata                                          ==##
##==  data/Rdata/TLR.Rdata                                          ==##
##==  data/Rdata/IL6.Rdata                                          ==##
##==  data/Rdata/TOP5.Rdata                                         ==##
##==  These files contain gDatP, a data frame with columns          ==##
##==  corresponding to patient id (id), ocMOF (grp), hour since     ==##
##==  injury (time), log2(gene expression+constant) (exp),          ==##
##==  probesets id (probeID).                                       ==##
##==  *log/information_paper.log                                    ==##
##==  Information provided in the paper                             ==##
##==                                                                ==##
##==Prepared by Chuen Seng Tan/John Storey/Keyur Desai              ==##
##==email: chuentan@princeton.edu/jstorey@princeton.edu/            ==##        
##==       kdesai@princeton.edu                                     ==##
##====================================================================##

##THe top probeset information
top500.can<-read.table("data/Ingenuity/canonical pathway.Top500.slope.txt",sep="\t",skip=2,header=TRUE,row.names=1)
top500.in<- read.table("data/Ingenuity/Top500slopeMapped.txt",sep="\t",skip=2,header=TRUE,row.names=1)
top500<-    read.table(file=paste("data/csv/res.",type1,"adj.csv",sep=""),sep=",")[1:500,]

##Getting WPEC and ocMOF
load(file=paste("data/Rdata/microarray/",type1,".Rdata",sep=""))

##==Filter the microarray data
##Remove the individuals due to quality reasons and death status
gDat<-gDat[,!(as.numeric(colnames(gDat))%in%c(8028773,25993337,33426107))]

##==The load the confounders
load(file=paste("data/Rdata/dat.cfr.Rdata"))

##================================================##
##==Order the patient_id according to microarray==##
##================================================##
ref<-colnames(gDat)
mat<-dat.cfr[,"patient_id"]
id<-match(ref,mat)
cDat<-dat.cfr[id,]
if(!all(cDat[,"patient_id"]==ref)) stop("confounders and gene expression not ordered correctly")

##========================##
##==THe MHC-II probesets==##
##========================##
pIds.mhc2<-c("204670_x_at","215193_x_at","209312_x_at","221491_x_at","208894_at","212998_x_at","211654_x_at","212999_x_at","209823_x_at","211656_x_at","213537_at","211991_s_at","211990_at","201137_s_at","212671_s_at","203932_at")

i<-6
##Information for the paper on MHC2 from DAVID
##This not the first time sink is used (i.e. it is at "code/ocMOF/Getocmof.R",)
sink(file="log/information_paper.log",append=TRUE,split=TRUE)
print("===============================")
print("6 : MHC2 from DAVID")
print(paste("No of probesets (DAVID):",length(pIds.mhc2)))
print(paste("No of genes (DAVID): 8, from David"))
tt<-which(top500[,1]%in%pIds.mhc2)
print(paste("No of probesets in the top 50",sum(tt<=50)))
print("===============================")
sink(file=NULL)
#top50<-top500[tt[tt<=50],]

##=================================##
##==THe p38MAPK canonical pathway==##
##=================================##
##THe MAPK14 probesets
pMAPK14<-c("211087_x_at","202530_at", "211561_x_at", "210449_x_at")

##This not the first time sink is used (i.e. it is at "code/ocMOF/Getocmof.R",)
sink(file="log/information_paper.log",append=TRUE,split=TRUE)
print("===============================")
print("MAPK14")
print(paste("No of probesets (top 500):",sum(top500[,1]%in%pMAPK14)))
print(paste("No of probesets (top 50):", sum(top500[1:50,1]%in%pMAPK14)))
print("===============================")
sink(file=NULL)


##Getting the probesets idenitied by ingenuity
i<-3
topi<-unlist(strsplit(as.character(top500.can[i,3]),split=","))
tmp<-rownames(top500.in)[which(top500.in[,2]%in%topi)]
#length(tmp)
#length(unique(top500.in[rownames(top500.in)%in%tmp,2]))
pIds<-tmp

##Information for the paper on p38MAPK
##This not the first time sink is used (i.e. it is at "code/ocMOF/Getocmof.R",)
sink(file="log/information_paper.log",append=TRUE,split=TRUE)
print("===============================")
print(paste(i,":",rownames(top500.can)[i]))
print(paste("No of probesets:",length(pIds)))
print(paste("No of genes:",length(topi)))
sink(file=NULL)

##Correlation of ocMOF and WPEC for top 50
pIds.ref<-as.character(top500[which(top500[1:50,1]%in%pIds),1]) 
cor.ref<-NULL
for(i.id in pIds.ref){
  cor.ref<-c(cor.ref,cor(unlist(gDat[i.id,]),cDat[,"cluster"],method="spearman"))
}
sign.ref<-unique(sign(cor.ref))

signID<-NULL
signval<-NULL
for(i in pIds.ref){
  i.id<-which(rownames(gDat)==i)
  i.signval<-cor(unlist(gDat[i.id,]),cDat[,"cluster"],method="spearman")
  i.sign<-sign(i.signval)
  
  j.val<-NULL
  for(j in pIds){
    j.id<-which(rownames(gDat)==j)
    j.val<-c(j.val,cor(unlist(gDat[j.id,]),cDat[,"cluster"],method="spearman"))
  }
  signval<-cbind(signval,j.val)
  signID<-cbind(signID,sign(j.val))
}
rownames(signID)<-rownames(signval)<-pIds
colnames(signID)<-colnames(signval)<-pIds.ref

#data.frame(names(pIds),pIds,signID)
if(all(apply(signID,1,function(x) length(unique(x)))==1)){
  signInfo<-ifelse(signID[,1]==1,1,-1)
}else{
  stop("There are differences in slopes signs")
}


##Remove the three probesets with inconsistent biological slope signs
pIds.p38<-pIds[!(pIds%in%c("209331_s_at","209199_s_at","1729_at"))]

##The situation after filtering the p38MAPK
##This not the first time sink is used (i.e. it is at "code/ocMOF/Getocmof.R",)
sink(file="log/information_paper.log",append=TRUE,split=TRUE)
print(paste("No of probesets (filtered):",length(pIds.p38)))
print(paste("No of genes (filtered):",length(unique(apply(matrix(pIds.p38,ncol=1),1,function(i,top500.in) top500.in[rownames(top500.in)%in%i,2],top500.in=top500.in)))))
tt<-which(top500[,1]%in%pIds.p38)
print(paste("No of probesets in the top 50 (filtered):",sum(tt<=50)))
top50<-top500[tt[tt<=50],]
print("Top 50")
print(top50)
print(apply(matrix(top50[,1],ncol=1),1,function(i,top500.in) top500.in[rownames(top500.in)%in%i,2],top500.in=top500.in))
print("Genes of p38MAPK (filtered):")
print(unique(apply(matrix(pIds.p38,ncol=1),1,function(i,top500.in) top500.in[rownames(top500.in)%in%i,2],top500.in=top500.in)))
print("===============================")
sink(file=NULL)

#print(data.frame(probesetID=pIds,gene=apply(matrix(pIds,ncol=1),1,function(i,top500.in) top500.in[rownames(top500.in)%in%i,2],top500.in=top500.in),sign.Info.ref.same=signInfo==sign.ref))
#signInfo<-(signInfo==sign.ref)



##=============================##
##==The TLR canonical pathway==##
##=============================##
##Getting the probesets idenitied by ingenuity
i<-2
topi<-unlist(strsplit(as.character(top500.can[i,3]),split=","))
tmp<-rownames(top500.in)[which(top500.in[,2]%in%topi)]
length(tmp)
length(unique(top500.in[rownames(top500.in)%in%tmp,2]))
pIds<-tmp

##Information for the paper on TLR
##This not the first time sink is used (i.e. it is at "code/ocMOF/Getocmof.R",)
sink(file="log/information_paper.log",append=TRUE,split=TRUE)
print("===============================")
print(paste(i,":",rownames(top500.can)[i]))
print(paste("No of probesets:",length(pIds)))
print(paste("No of genes:",length(topi)))
sink(file=NULL)

##Correlation of ocMOF and WPEC for top 50
pIds.ref<-as.character(top500[which(top500[1:50,1]%in%pIds),1]) 
cor.ref<-NULL
for(i.id in pIds.ref){
  cor.ref<-c(cor.ref,cor(unlist(gDat[i.id,]),cDat[,"cluster"],method="spearman"))
}
sign.ref<-unique(sign(cor.ref))

signID<-NULL
signval<-NULL
for(i in pIds.ref){
  i.id<-which(rownames(gDat)==i)
  i.signval<-cor(unlist(gDat[i.id,]),cDat[,"cluster"],method="spearman")
  i.sign<-sign(i.signval)
  
  j.val<-NULL
  for(j in pIds){
    j.id<-which(rownames(gDat)==j)
    j.val<-c(j.val,cor(unlist(gDat[j.id,]),cDat[,"cluster"],method="spearman"))
  }
  signval<-cbind(signval,j.val)
  signID<-cbind(signID,sign(j.val))
}
rownames(signID)<-rownames(signval)<-pIds
colnames(signID)<-colnames(signval)<-pIds.ref

#data.frame(names(pIds),pIds,signID)
if(all(apply(signID,1,function(x) length(unique(x)))==1)){
  signInfo<-ifelse(signID[,1]==1,1,-1)
}else{
  stop("There is differences in slopes signs")
}

#print(data.frame(probesetID=pIds,gene=apply(matrix(pIds,ncol=1),1,function(i,top500.in) top500.in[rownames(top500.in)%in%i,2],top500.in=top500.in),sign.Info.ref.same=signInfo==sign.ref))
#signInfo<-(signInfo==sign.ref)

##Remove the one probesets with inconsistent biological slope signs
pIds.tlr<-pIds[!(pIds%in%c("229664_at"))]

##The situation after filtering TLR
##This not the first time sink is used (i.e. it is at "code/ocMOF/Getocmof.R",)
sink(file="log/information_paper.log",append=TRUE,split=TRUE)
print(paste("No of probesets (filtered):",length(pIds.tlr)))
print(paste("No of genes (filtered):",length(unique(apply(matrix(pIds.tlr,ncol=1),1,function(i,top500.in) top500.in[rownames(top500.in)%in%i,2],top500.in=top500.in)))))
#tt<-which(top500[,1]%in%pIds.tlr)
#print(paste("No of probesets in the top 50 (filtered):",sum(tt<=50)))
#top50<-top500[tt[tt<=50],]
#print("Top 50")
#print(top50)
#print(apply(matrix(top50[,1],ncol=1),1,function(i,top500.in) top500.in[rownames(top500.in)%in%i,2],top500.in=top500.in))
print("===============================")
sink(file=NULL)



##==============================##
##==THe IL-6 canonical pathway==##
##==============================##
##Getting the probesets idenitied by ingenuity
i<-4
topi<-unlist(strsplit(as.character(top500.can[i,3]),split=","))
tmp<-rownames(top500.in)[which(top500.in[,2]%in%topi)]
length(tmp)
length(unique(top500.in[rownames(top500.in)%in%tmp,2]))
pIds<-tmp

##Information for the paper
##This not the first time sink is used (i.e. it is at "code/ocMOF/Getocmof.R",)
sink(file="log/information_paper.log",append=TRUE,split=TRUE)
print("===============================")
print(paste(i,":",rownames(top500.can)[i]))
print(paste("No of probesets:",length(pIds)))
print(paste("No of genes:",length(topi)))
sink(file=NULL)

##Correlation of ocMOF and WPEC for top 50
pIds.ref<-as.character(top500[which(top500[1:50,1]%in%pIds),1]) 
cor.ref<-NULL
for(i.id in pIds.ref){
  cor.ref<-c(cor.ref,cor(unlist(gDat[i.id,]),cDat[,"cluster"],method="spearman"))
}
sign.ref<-unique(sign(cor.ref))

signID<-NULL
signval<-NULL
for(i in pIds.ref){
  i.id<-which(rownames(gDat)==i)
  i.signval<-cor(unlist(gDat[i.id,]),cDat[,"cluster"],method="spearman")
  i.sign<-sign(i.signval)
  
  j.val<-NULL
  for(j in pIds){
    j.id<-which(rownames(gDat)==j)
    j.val<-c(j.val,cor(unlist(gDat[j.id,]),cDat[,"cluster"],method="spearman"))
  }
  signval<-cbind(signval,j.val)
  signID<-cbind(signID,sign(j.val))
}
rownames(signID)<-rownames(signval)<-pIds
colnames(signID)<-colnames(signval)<-pIds.ref

#data.frame(names(pIds),pIds,signID)
if(all(apply(signID,1,function(x) length(unique(x)))==1)){
  signInfo<-ifelse(signID[,1]==1,1,-1)
}else{
  stop("There is differences in slopes signs")
}

#print(data.frame(probesetID=pIds,gene=apply(matrix(pIds,ncol=1),1,function(i,top500.in) top500.in[rownames(top500.in)%in%i,2],top500.in=top500.in),sign.Info.ref.same=signInfo==sign.ref))
#signInfo<-(signInfo==sign.ref)

##Remove the one probeset with inconsistent biological slope signs
pIds.il6<-pIds[!(pIds%in%c("229664_at"))]

##The situation after filtering IL6
##This not the first time sink is used (i.e. it is at "code/ocMOF/Getocmof.R",)
sink(file="log/information_paper.log",append=TRUE,split=TRUE)
print(paste("No of probesets (filtered):",length(pIds.il6)))
print(paste("No of genes (filtered):",length(unique(apply(matrix(pIds.il6,ncol=1),1,function(i,top500.in) top500.in[rownames(top500.in)%in%i,2],top500.in=top500.in)))))
#tt<-which(top500[,1]%in%pIds.il6)
#print(paste("No of probesets in the top 50 (filtered):",sum(tt<=50)))
#top50<-top500[tt[tt<=50],]
#print("Top 50")
#print(top50)
#print(apply(matrix(top50[,1],ncol=1),1,function(i,top500.in) top500.in[rownames(top500.in)%in%i,2],top500.in=top500.in))
print("===============================")
sink(file=NULL)




##===============================##
##==THe top 5 canonical pathway==##
##===============================##
##Getting the probesets idenitied by ingenuity
i<-5
topi<-unlist(strsplit(as.character(top500.can[i,3]),split=","))
tmp<-rownames(top500.in)[which(top500.in[,2]%in%topi)]
length(tmp)
length(unique(top500.in[rownames(top500.in)%in%tmp,2]))
pIds<-tmp

##Information for the paper on the top 5 canonical pathway
##This not the first time sink is used (i.e. it is at "code/ocMOF/Getocmof.R",)
sink(file="log/information_paper.log",append=TRUE,split=TRUE)
print("===============================")
print(paste(i,":",rownames(top500.can)[i]))
print(paste("No of probesets:",length(pIds)))
print(paste("No of genes:",length(topi)))
sink(file=NULL)

##Correlation of ocMOF and WPEC for top 50
pIds.ref<-as.character(top500[which(top500[1:50,1]%in%pIds),1]) 
pIds.ref<-pIds.ref[!(pIds.ref%in%pMAPK14)]
cor.ref<-NULL
for(i.id in pIds.ref){
  cor.ref<-c(cor.ref,cor(unlist(gDat[i.id,]),cDat[,"cluster"],method="spearman"))
}
sign.ref<-unique(sign(cor.ref))

signID<-NULL
signval<-NULL
for(i in pIds.ref){
  i.id<-which(rownames(gDat)==i)
  i.signval<-cor(unlist(gDat[i.id,]),cDat[,"cluster"],method="spearman")
  i.sign<-sign(i.signval)
  
  j.val<-NULL
  for(j in pIds){
    j.id<-which(rownames(gDat)==j)
    j.val<-c(j.val,cor(unlist(gDat[j.id,]),cDat[,"cluster"],method="spearman"))
  }
  signval<-cbind(signval,j.val)
  signID<-cbind(signID,sign(j.val))
}
rownames(signID)<-rownames(signval)<-pIds
colnames(signID)<-colnames(signval)<-pIds.ref

#data.frame(names(pIds),pIds,signID)
if(all(apply(signID,1,function(x) length(unique(x)))==1)){
  signInfo<-ifelse(signID[,1]==1,1,-1)
}else{
  stop("There is differences in slopes signs")
}

#print(data.frame(probesetID=pIds,gene=apply(matrix(pIds,ncol=1),1,function(i,top500.in) top500.in[rownames(top500.in)%in%i,2],top500.in=top500.in),sign.Info.ref.same=signInfo==sign.ref))
#signInfo<-(signInfo==sign.ref)

##Remove the one probeset with inconsistent biological slope signs
pIds.top5<-pIds[!(pIds%in%c("210449_x_at","211087_x_at","211561_x_at","202530_at","229664_at","201502_s_at","201604_s_at"))]
#print(data.frame(probesetID=pIds.top5,gene=apply(matrix(pIds.top5,ncol=1),1,function(i,top500.in) top500.in[rownames(top500.in)%in%i,2],top500.in=top500.in),sign.Info.ref.same=signInfo[apply(matrix(pIds.top5,ncol=1),1,function(i,ref) which(names(ref)%in%i,2),ref=signInfo)]))

##The situation after filtering Top5
##This not the first time sink is used (i.e. it is at "code/ocMOF/Getocmof.R",)
sink(file="log/information_paper.log",append=TRUE,split=TRUE)
print(paste("No of probesets (filtered):",length(pIds.top5)))
print(paste("No of genes (filtered):",length(unique(apply(matrix(pIds.top5,ncol=1),1,function(i,top500.in) top500.in[rownames(top500.in)%in%i,2],top500.in=top500.in)))))
#tt<-which(top500[,1]%in%pIds.top5)
#print(paste("No of probesets in the top 50 (filtered):",sum(tt<=50)))
#top50<-top500[tt[tt<=50],]
#print("Top 50")
#print(top50)
#print(apply(matrix(top50[,1],ncol=1),1,function(i,top500.in) top500.in[rownames(top500.in)%in%i,2],top500.in=top500.in))
print("===============================")
sink(file=NULL)

##Filtered set of probsets associated with each gene set
save(pIds.mhc2,pIds.p38,pIds.tlr,pIds.il6,pIds.top5,
     file="data/Rdata/probesets.CanonicalPathway.Rdata")

##Information regarding microarray id, patient id, hours and batch, and the probesets for 126 patients
load(file=paste("data/Rdata/microarray/",txt.info,".Rdata",sep=""))
id.126<-which(patient.id%in%colnames(gDat))
Xraw<-Xraw[,id.126]
patient.id<-patient.id[id.126]
microarray.id<-microarray.id[id.126]
hours.si<-hours.si[id.126]
ocMOF.id<-cDat[match(patient.id,cDat[,"patient_id"]),"cluster"]
rm(list=c("id.126","gDat"))

pID<-pIds.mhc2
Xraw.id<-which(rownames(Xraw)%in%pID)
gDatP<-data.frame(id=rep(patient.id,times=length(pID)),
                  grp=rep(ocMOF.id,times=length(pID)),
                  time=rep(hours.si,times=length(pID)),
                  exp=log2(c(t(Xraw[Xraw.id,]))+constant),
                  probeID=rep(rownames(Xraw)[Xraw.id],each=length(patient.id)),
                  stringsAsFactors=FALSE)
save(gDatP,file="data/Rdata/MHC2.Rdata")

pID<-pIds.p38
Xraw.id<-which(rownames(Xraw)%in%pID)
gDatP<-data.frame(id=rep(patient.id,times=length(pID)),
                  grp=rep(ocMOF.id,times=length(pID)),
                  time=rep(hours.si,times=length(pID)),
                  exp=log2(c(t(Xraw[Xraw.id,]))+constant),
                  probeID=rep(rownames(Xraw)[Xraw.id],each=length(patient.id)),
                  stringsAsFactors=FALSE)
save(gDatP,file="data/Rdata/P38.Rdata")

pID<-pIds.tlr
Xraw.id<-which(rownames(Xraw)%in%pID)
gDatP<-data.frame(id=rep(patient.id,times=length(pID)),
                  grp=rep(ocMOF.id,times=length(pID)),
                  time=rep(hours.si,times=length(pID)),
                  exp=log2(c(t(Xraw[Xraw.id,]))+constant),
                  probeID=rep(rownames(Xraw)[Xraw.id],each=length(patient.id)),
                  stringsAsFactors=FALSE)
save(gDatP,file="data/Rdata/TLR.Rdata")

pID<-pIds.il6
Xraw.id<-which(rownames(Xraw)%in%pID)
gDatP<-data.frame(id=rep(patient.id,times=length(pID)),
                  grp=rep(ocMOF.id,times=length(pID)),
                  time=rep(hours.si,times=length(pID)),
                  exp=log2(c(t(Xraw[Xraw.id,]))+constant),
                  probeID=rep(rownames(Xraw)[Xraw.id],each=length(patient.id)),
                  stringsAsFactors=FALSE)
save(gDatP,file="data/Rdata/IL6.Rdata")

pID<-pIds.top5
Xraw.id<-which(rownames(Xraw)%in%pID)
gDatP<-data.frame(id=rep(patient.id,times=length(pID)),
                  grp=rep(ocMOF.id,times=length(pID)),
                  time=rep(hours.si,times=length(pID)),
                  exp=log2(c(t(Xraw[Xraw.id,]))+constant),
                  probeID=rep(rownames(Xraw)[Xraw.id],each=length(patient.id)),
                  stringsAsFactors=FALSE)
save(gDatP,file="data/Rdata/TOP5.Rdata")

