##====================================================================##
##==Purpose: To clean up the clinical data and keep the first 6 days==##
##==                                                                ==##
##==Input:                                                          ==##
##==   data/Rdata/clinDatInfo12_250hr.Rdata                         ==##
##==                                                                ==##
##==Output:                                                         ==##
##==   data/Rdata/clinDatInfo12_250hr(clean).Rdata                  ==##
##==   *clinDat, a dataframe with columns corresponding to clinical ==##
##==   variables and rows the patients                              ==##
##==   *pcall, a vector indicating the percentage of missing in the ==##
##==   each column.                                                 ==##             
##==   *datType, a vector indicating which data source the clinical ==##
##==   variable came from.                                          ==##
##==                                                                ==##
##==Additional info:                                                ==##
##==   For clinical variables that have "LATE_RECOV" convert 444    ==##
##==   and 555 to NAs:                                              ==##
##==   - 444 means died in non-recovery for this organ system.      ==##
##==   - 555 means no recovery by day 28.                           ==##
##==                                                                ==##
##==Prepared by Chuen Seng Tan/John Storey/Keyur Desai              ==##
##==email: chuentan@princeton.edu/jstorey@princeton.edu/            ==##
##==       kdesai@princeton.edu                                     ==##
##====================================================================##

##Load the clinical variables
load(file="data/Rdata/clinDatInfo12_250hr.Rdata")

##Ensuring blanks "" and " " are NAs [Note that]
clinDatNV[clinDatNV==""]<-NA
clinDatNVC[clinDatNVC==""]<-NA
clinDatTV[clinDatTV==""]<-NA
clinDatMOF[clinDatMOF==""]<-NA
clinDatMOF[clinDatMOF==" "]<-NA

##Replace 444 and 555 with NA, which indicate those who died and did not recovery after day 28
idrm<-grep("LATE_RECOV",colnames(clinDatNVC))
summary(clinDatNVC[,idrm])
if(length(idrm)>0){
  na.id<-clinDatNVC[,idrm]==444 | clinDatNVC[,idrm]==555
  clinDatNVC[,idrm][na.id]<-NA
}

##Filter out those with >40% NA entries
naNo<-apply(clinDatNV,2,function(x) (sum(is.na(x))/length(x))>0.4)
clinDatNV<-clinDatNV[,!naNo]

naNo<-apply(clinDatNVC,2,function(x) (sum(is.na(x))/length(x))>0.4)
clinDatNVC<-clinDatNVC[,!naNo]

naNo<-apply(clinDatRES,2,function(x) (sum(is.na(x))/length(x))>0.4)
clinDatRES<-clinDatRES[,!naNo]

naNo<-apply(clinDatTV,2,function(x) (sum(is.na(x))/length(x))>0.4)
clinDatTV<-clinDatTV[,!naNo]

naNo<-apply(clinDatBC,2,function(x) (sum(is.na(x))/length(x))>0.4)
clinDatBC<-clinDatBC[,!naNo]

naNo<-apply(clinDatMOF,2,function(x) (sum(is.na(x))/length(x))>0.4)
clinDatMOF<-clinDatMOF[,!naNo]

##Remove those that have only 1 value
uNo<-apply(clinDatNV,2,function(x) sum(!(unique(x)%in%NA))==1)
clinDatNV<-clinDatNV[,!uNo]

uNo<-apply(clinDatNVC,2,function(x) sum(!(unique(x)%in%NA))==1)
clinDatNVC<-clinDatNVC[,!uNo]

uNo<-apply(clinDatRES,2,function(x) sum(!(unique(x)%in%NA))==1)
clinDatRES<-clinDatRES[,!uNo]

uNo<-apply(clinDatTV,2,function(x) sum(!(unique(x)%in%NA))==1)
clinDatTV<-clinDatTV[,!uNo]

uNo<-apply(clinDatBC,2,function(x) sum(!(unique(x)%in%NA))==1)
clinDatBC<-clinDatBC[,!uNo]

uNo<-apply(clinDatMOF,2,function(x) sum(!(unique(x)%in%NA))==1)
clinDatMOF<-clinDatMOF[,!uNo]

##Remove those clinical variable > 6 days since death or discharge date is 6 days
summary(clinDatNV[,"DEATH_OR_DISCHARGE_DAY_SNC_INJ"])
##The number of clinical variables with p-value<0.05
valid.TV<-sapply(strsplit(colnames(clinDatTV),split="\\."),function(x) as.numeric(x[2])<=6)
clinDatTV<-clinDatTV[,c(1,which(valid.TV==TRUE))]
valid.BC<-sapply(strsplit(colnames(clinDatBC),split="\\."),function(x) as.numeric(x[2])<=6)
clinDatBC<-clinDatBC[,c(1,which(valid.BC==TRUE))]
valid.MOF<-sapply(strsplit(colnames(clinDatMOF),split="\\."),function(x) as.numeric(x[2])<=6)
clinDatMOF<-clinDatMOF[,c(1,which(valid.MOF==TRUE))]
valid.RES<-sapply(strsplit(colnames(clinDatRES),split="\\:"),function(x) x[2]==" 0 - 6")
clinDatRES<-clinDatRES[,c(1,which(valid.RES==TRUE))]

##For clinDatNV, remove SNC_SS
id<-grep("SNC_SS",colnames(clinDatNV))
clinDatNV<-clinDatNV[,-id]

##Merge ALL clinical data together
if(!all(clinDatNV[,1]==clinDatNVC[,1])) stop("Data do not match")
if(!all(clinDatNV[,1]==clinDatRES[,1])) stop("Data do not match")
if(!all(clinDatNV[,1]==clinDatTV[,1]))  stop("Data do not match")
if(!all(clinDatNV[,1]==clinDatBC[,1]))  stop("Data do not match")
if(!all(clinDatNV[,1]==clinDatMOF[,1])) stop("Data do not match")

##Remove INSULIN_REQ [TV] because MOF_INS2 [MOF] are highly correlated
idrm<-grep("INSULIN_REQ",colnames(clinDatTV))
if(length(idrm)>0){
  clinDatTV<-clinDatTV[,-idrm]
}
##Remove RECOVERY because some of it is inferred yes or no
idrm<-grep("RECOVERY",colnames(clinDatTV))
if(length(idrm)>0){
  clinDatTV<-clinDatTV[,-idrm]
}

clinDat<-data.frame(clinDatNV,clinDatNVC[,-1],clinDatTV[,-1],clinDatMOF[,-1],clinDatBC[,-1],clinDatRES[,-1])
datType<-c("id",rep("clinDatNV",ncol(clinDatNV)-1),rep("clinDatNVC",ncol(clinDatNVC)-1),rep("clinDatTV",ncol(clinDatTV)-1),
           rep("clinDatMOF",ncol(clinDatMOF)-1),rep("clinDatBC",ncol(clinDatBC)-1),rep("clinDatRES",ncol(clinDatRES)-1))
pcall<-apply(clinDat,2,function(x) sum(!is.na(x))/length(x))

save(clinDat,pcall,datType,file="data/Rdata/clinDatInfo12_250hr(clean).Rdata")

