##====================================================================##
##==Purpose: Reasons for removing the first 12 hours expression     ==##
##==                                                                ==##
##==Input:                                                          ==##
##==  *txt.info                                                     ==##
##==  Specifying name of the *.txt file which contains the          ==##
##==  microarrays id.                                               ==##
##==  *"data/Rdata/microarray/",txt.info,".Rdata"                   ==##
##==  -Xraw: A matrix with rows corresponding to probesets, and the ==##
##==         columns the microarrays of patients at different time  ==##
##==         points.                                                ==##
##==  The follow R objects are used to identify the columns of Xraw.==##  
##==  -patient.id: A vector of patients ID                          ==##
##==  -microarray.id: A vector of microarray ID                     ==##
##==  -batch: A vector of batch ID                                  ==##
##==  -hours.si: A vector of the timing which the microarray was    ==##
##==             collected (in hours).                              ==##
##==  *type1                                                        ==##
##==  A string indicating the gene expression of interest           ==##
##==  *"data/Rdata/microarray/",type1,".Rdata"                      ==##
##==  gDat, a matrix with columns corresponding to patients and     ==##
##==  rows the probests. The entries are the related to gene        ==##
##==  expressions.                                                  ==##
##==  *Additional parameters                                        ==##
##==  -min_hours, e.g. 12                                           ==##
##==  -max_hours, e.g. 250                                          ==## 
##==                                                                ==##
##==Figures for paper:                                              ==##
##==  *Illustrating the reason for exluding the first 12 hours      ==##
##==  plots/forpaper/Supplementary_Figure5_first12hr.pdf            ==##
##==  *The boxplot of the MSD as we remove progressively more obs   ==##
##==  plots/forpaper/Supplementary_Figure6_MSD.pdf                  ==##
##==                                                                ==##
##==Sub-routines:                                                   ==##
##==  *Function to generate the slope matrix                        ==##
##==  code/2_normalization/mtx_linfit.R                             ==##
##==  *To compute WPEC                                              ==##
##==  code/2_normalization/ComputeWPEC.R                            ==##
##==                                                                ==##
##==Output:                                                         ==##
##==  data/Rdata/DWPEC.Rdata                                        ==##
##==  data/Rdata/p.value.DWPEC.Rdata				    ==##
##==  "data/Rdata/probeSlopeOffset[",hr,"_250].Rdata"               ==##
##==  "data/Rdata/probeLoessFit[",hr,"_250].Rdata"                  ==##
##==  data/Rdata/mse.all.Rdata                                      ==##
##==                                                                ==##
##==R library: qvalue                                               ==##
##==                                                                ==##
##==Prepared by Chuen Seng Tan/John Storey/Keyur Desai              ==##
##==email: chuentan@princeton.edu/jstorey@princeton.edu/            ==##
##==       kdesai@princeton.edu                                     ==##
##====================================================================##


library(qvalue)

##=================================================##
##==Getting the DWPEC for the supplementary paper==##
##=================================================##
##Generate the WPEC matrix from 0-250 hours
txt.info<-"microarrayID250.168.batch"
constant<-10
min_hours<-0
max_hours<-250
#Summary method (0=LR Slope, 1=LR output intercept, 2=Mean value)
method<-0
source("code/2_normalization/ComputeWPEC.R",local=TRUE)
rm(list=ls())


##==Compute DWPEC=WPEC(0-250hr)-WPEC(12-250hr)
txt.info<-"microarrayID250.168.batch"
min_hours<-12
max_hours<-250
##WPEC(12-250), tS
tS<-paste(txt.info," Slope (",min_hours,"-",max_hours,")",sep="")
##WPEC(0-250), tS
min_hours<-0
tS0<-paste(txt.info," Slope (",min_hours,"-",max_hours,")",sep="")

for(id in c("tS","tS0")){
	type1<-get(id)
	load(file=paste("data/Rdata/microarray/",type1,".Rdata",sep=""))
    ##Analyze 126 patients
	gDat<-gDat[,!(as.numeric(colnames(gDat))%in%c(25993337,33426107,8028773))]
	assign(id,gDat)
}

##Order the patient_id according to WPEC(12-250) for both tS and tS0
ref<-colnames(tS)
mat<-colnames(tS0)
id<-match(ref,mat)
tS0<-tS0[,id]
if(!all(colnames(tS0)==colnames(tS))) stop("Columns of WPEC for 0-250 and 12-250 do not match")
if(!all(rownames(tS0)==rownames(tS))) stop("Rows of WPEC for 0-250 and 12-250 do not match")

##Differences in slope (DWPEC)
d.tS<-tS-tS0
rm(tS0,tS)
load(file=paste("data/Rdata/microarray/",txt.info,".Rdata",sep=""))
rm(microarray.id,batch,Xraw)
##Exclude indivduals where WPEC(0-250) & WPEC(12-250) is the same (i.e. no change)
id<-which(apply(d.tS==0,2,all))
if(!all(hours.si[patient.id%in%(colnames(d.tS)[id])]>=12)) stop("DWPEC is inconsistent. Expecting these patients to have microarrays>=12")
d.tS<-d.tS[,-id]
save(d.tS,file="data/Rdata/DWPEC.Rdata")

##The one sample t-test on DWPEC
p.val<-apply(d.tS,1,function(x) t.test(x=x)$p.value)
qval<-qvalue(p.val)
##This not the first time sink is used (i.e. it is at "code/ocMOF/Getocmof.R",)
sink(file="log/information_paper.log",append=TRUE,split=TRUE)
print("===============================================================================================")
print("The estimated proportion of probesets with either flat or had reasonably montonic trajectories:")
print(round(qval$pi0*100))
print(paste("The number of patients used for DWPEC analysis is",ncol(d.tS)))
print("===============================================================================================")
sink(file=NULL)
save(p.val,file="data/Rdata/p.value.DWPEC.Rdata")
rm(list=ls())



##===============================================================##
##==Generate the boxplots of MSE for the supplementary appendix==##
##===============================================================##

##==Filtering the data to the number used for computing DWPEC
#d.tS,
load(file="data/Rdata/DWPEC.Rdata")
#p.val,
load(file="data/Rdata/p.value.DWPEC.Rdata")
txt.info<-"microarrayID250.168.batch"
#patient.id,microarray.id,hours.si,batch,Xraw,
load(file=paste("data/Rdata/microarray/",txt.info,".Rdata",sep=""))

##Restrict the number of patients used to those used in computing DWPEC
valid<-which(patient.id%in%colnames(d.tS))     
patient.id<-patient.id[valid]
Xraw<-Xraw[,valid]
hours.si<-hours.si[valid]
microarray.id<-microarray.id[valid]
batch<-batch[valid]
if(!(length(unique(patient.id))==ncol(d.tS)))  stop("The expression data does not have 108 patients")

##Getting the top and bottom 5000 probesets
ntop<-5000
id.sig<-order(p.val)[1:ntop]
id.ns<-rev(order(p.val))[1:ntop]


##==Get the linear fit for each patient at various time window:
##==0-250, 4-250, 8-250, 12-250   
##Function to compute the Slope (WPEC)/Intercept, mtx.linfit
source("code/2_normalization/mtx_linfit.R",local=TRUE)
constant<-10
##Consolidate the MSE across the various time window
mse.all<-NULL
for(hr in c(0,4,8,12)){
  
  print(paste("##==Hour",hr,"to 250"))
  print("Performing linear regression")
  ##Linear fit for each patient at various time window	
  probeSlope<-matrix(NA,ncol=ncol(d.tS),nrow=nrow(Xraw))
  probeOffset<-matrix(NA,ncol=ncol(d.tS),nrow=nrow(Xraw))
  for(k in 1:ncol(d.tS)){

    valid<-which(patient.id==colnames(d.tS)[k] & hours.si>=hr)
    #Linear regression,slope:
	probeSlope[,k] <- mtx.linfit(log2(Xraw[,valid]+constant),hours.si[valid],"slope");
	#Linear regression, offset/intercept:
	probeOffset[,k]<- mtx.linfit(log2(Xraw[,valid]+constant),hours.si[valid],"offset");
	cat("\r","Patient:",k)
  }
  cat("\n")
  save(probeSlope,probeOffset,file=paste("data/Rdata/probeSlopeOffset[",hr,"_250].Rdata",sep=""))


  print("Performing loess fit")  	
  time1<-min(unique(hours.si[hours.si>=hr]))
  time2<-max(unique(hours.si))
  time<-seq(time1,time2,len=300)
	
  dat.pr.sig<-NULL
  dat.pr.ns<-NULL
  for(j in 1:length(id.sig)){
	  
    ##The significant probesets from DWPEC  
    i<-id.sig[j]
	dat<-data.frame(exp=log2(Xraw[i,]+constant),time=hours.si)
	dat<-dat[dat[,2]>=hr,]
	dat.lo <- loess(exp ~ time, dat)
	dat.pr<-cbind(time,predict(dat.lo,time))
	if(all(time==dat.pr[,1]))
	  dat.pr.sig<-cbind(dat.pr.sig,dat.pr[,2])

	  
    ##The non-significant probesets from DWPEC  
    i<-id.ns[j]
	dat<-data.frame(exp=log2(Xraw[i,]+constant),time=hours.si)
	dat<-dat[dat[,2]>=hr,]
	dat.lo <- loess(exp ~ time, dat)
	dat.pr<-cbind(time,predict(dat.lo,time))
	if(all(time==dat.pr[,1]))
	  dat.pr.ns<-cbind(dat.pr.ns,dat.pr[,2])
	cat("\r","Probe:",j)  
  }
  cat("\n")	
  save(time,dat.pr.sig,dat.pr.sig,file=paste("data/Rdata/probeLoessFit[",hr,"_250].Rdata",sep=""))

  ##GET the MSE for the significant probests (hr-250)
  par.lm.sig<-cbind(rowMeans(probeOffset[id.sig,]),rowMeans(probeSlope[id.sig,]))
  lm.est.sig<-model.matrix(~time,data=data.frame(time=time))%*%t(par.lm.sig)
  mse.sig<-colMeans((lm.est.sig-dat.pr.sig)^2)
	
  ##GET the MSE for the insignificant probests (hr-250)
  par.lm.ns<-cbind(rowMeans(probeOffset[id.ns,]),rowMeans(probeSlope[id.ns,]))
  lm.est.ns<-model.matrix(~time,data=data.frame(time=time))%*%t(par.lm.ns)
  mse.ns<-colMeans((lm.est.ns-dat.pr.ns)^2)
  save(mse.ns,mse.sig,time,file=paste("data/Rdata/mse[",hr,"_250].Rdata",sep=""))
  
  ##Consolidate the MSE across various windowa
  mse.all<-rbind(mse.all,data.frame(c(mse.ns,mse.sig),rep(c(paste("ns: ",hr,"-250",sep=""),paste("sig: ",hr,"-250",sep="")),each=length(mse.ns))))  
		
}
##Name the columns of mse.all
colnames(mse.all)<-c("mse","type")
save(mse.all,file=paste("data/Rdata/mse.all.Rdata",sep=""))

##==Boxplot of the MSD for the top and bottom probesets
pdf("plots/forpaper/Supplementary_Figure6_MSD.pdf",width=10,height=7)
par(mfrow=c(1,1))
boxplot(mse~type,data=mse.all,ylab=
        "Mean squared difference between loess curve and linear line",
        ylim=c(0,0.7),axes=FALSE)
axis(2,las=1)
label<-NULL
for(hr in c(0,4,8,12)){
	label<-c(label,paste("ns: ",hr,"-250",sep=""),paste("sig: ",hr,"-250",sep=""))
}
axis(1,at=c(1:8),labels=label)
box()
dev.off()



##==========================================================##
##==Illustrate the reason for excluding the first 12 hours==##
##==========================================================##
pdf("plots/forpaper/Supplementary_Figure5_first12hr.pdf")
par(mfcol=c(2,2))
for(hr in c(0,12)){
  ##==0-250
  load(file=paste("data/Rdata/probeSlopeOffset[",hr,"_250].Rdata",sep=""))
  load(file=paste("data/Rdata/probeLoessFit[",hr,"_250].Rdata",sep=""))

  ##==The non-significant
  par.lm.ns<-cbind(rowMeans(probeOffset[id.ns,]),rowMeans(probeSlope[id.ns,]))

  j<-1
  i<-id.ns[j]
  dat<-data.frame(exp=log2(Xraw[i,]+10),time=hours.si) 
  dat<-dat[dat[,2]>=hr,]
  if(hr==0) ylim.ns<-range(dat.pr.ns[,j],rbind(c(1,0),c(1,250))%*%par.lm.ns[j,],dat[,1])
  plot(0,xlim=range(dat[,2]),ylim=ylim.ns,type="n",
       xlab="Hours since injury",ylab="log expression",
	   cex.lab=1.5,cex.axis=1.4,axes=FALSE)
  axis(2,las=1)
  axis(1)
  box()
  for(k in colnames(d.tS)){
    valid<-which(patient.id[hours.si>=hr]==k)
    lines(dat[valid,2:1],col="gray")
  }
  lines(time,dat.pr.ns[,j],lwd=3)
  lines(c(0,250),rbind(c(1,0),c(1,250))%*%par.lm.ns[j,],col=2,lwd=3)

  ##==The signficant
  par.lm.sig<-cbind(rowMeans(probeOffset[id.sig,]),rowMeans(probeSlope[id.sig,]))

  j<-1
  i<-id.sig[j]
  dat<-data.frame(exp=log2(Xraw[i,]+10),time=hours.si) 
  dat<-dat[dat[,2]>=hr,]
  if(hr==0) ylim.sig<-range(dat.pr.sig[,j],rbind(c(1,0),c(1,250))%*%par.lm.sig[j,],dat[,1])	
  plot(0,xlim=range(dat[,2]),ylim=ylim.sig,type="n",
       xlab="Hours since injury",ylab="log expression",
	   cex.lab=1.5,cex.axis=1.4,axes=FALSE)
  axis(2,las=1)
  axis(1)
  box()
  for(k in colnames(d.tS)){
    valid<-which(patient.id[hours.si>=hr]==k)
    lines(dat[valid,2:1],col="gray") 
  }
  lines(time,dat.pr.sig[,j],lwd=3)
  lines(c(0,250),rbind(c(1,0),c(1,250))%*%par.lm.sig[j,],col=2,lwd=3)

}

dev.off()
